# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'channel_stats.ui'
##
## Created by: Qt User Interface Compiler version 6.7.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractScrollArea, QApplication, QComboBox, QGridLayout,
    QGroupBox, QLabel, QScrollArea, QSizePolicy,
    QSpacerItem, QTextEdit, QVBoxLayout, QWidget)

class Ui_ChannelStats(object):
    def setupUi(self, ChannelStats):
        if not ChannelStats.objectName():
            ChannelStats.setObjectName(u"ChannelStats")
        ChannelStats.resize(315, 852)
        font = QFont()
        font.setFamilies([u"Roboto Condensed"])
        font.setPointSize(10)
        ChannelStats.setFont(font)
        self.verticalLayout_2 = QVBoxLayout(ChannelStats)
        self.verticalLayout_2.setSpacing(1)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.scrollArea = QScrollArea(ChannelStats)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 309, 846))
        self.verticalLayout = QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setSpacing(1)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(1, 1, 1, 1)
        self.gridLayout = QGridLayout()
        self.gridLayout.setSpacing(1)
        self.gridLayout.setObjectName(u"gridLayout")
        self.visible_group = QGroupBox(self.scrollAreaWidgetContents)
        self.visible_group.setObjectName(u"visible_group")
        self.gridLayout_3 = QGridLayout(self.visible_group)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setHorizontalSpacing(1)
        self.gridLayout_3.setVerticalSpacing(1)
        self.gridLayout_3.setContentsMargins(1, 1, 1, 1)
        self.visible_start = QLabel(self.visible_group)
        self.visible_start.setObjectName(u"visible_start")
        self.visible_start.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_start, 0, 1, 1, 1)

        self.unit7 = QLabel(self.visible_group)
        self.unit7.setObjectName(u"unit7")
        self.unit7.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.unit7, 8, 2, 1, 1)

        self.label_10 = QLabel(self.visible_group)
        self.label_10.setObjectName(u"label_10")

        self.gridLayout_3.addWidget(self.label_10, 4, 0, 1, 1)

        self.label_35 = QLabel(self.visible_group)
        self.label_35.setObjectName(u"label_35")

        self.gridLayout_3.addWidget(self.label_35, 7, 0, 1, 1)

        self.unit5 = QLabel(self.visible_group)
        self.unit5.setObjectName(u"unit5")
        self.unit5.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.unit5, 3, 2, 1, 1)

        self.visible_stop = QLabel(self.visible_group)
        self.visible_stop.setObjectName(u"visible_stop")
        self.visible_stop.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_stop, 1, 1, 1, 1)

        self.label_19 = QLabel(self.visible_group)
        self.label_19.setObjectName(u"label_19")

        self.gridLayout_3.addWidget(self.label_19, 8, 0, 1, 1)

        self.label_9 = QLabel(self.visible_group)
        self.label_9.setObjectName(u"label_9")

        self.gridLayout_3.addWidget(self.label_9, 3, 0, 1, 1)

        self.visible_rms = QLabel(self.visible_group)
        self.visible_rms.setObjectName(u"visible_rms")
        self.visible_rms.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_rms, 6, 1, 1, 1)

        self.unit17 = QLabel(self.visible_group)
        self.unit17.setObjectName(u"unit17")
        self.unit17.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.unit17, 7, 2, 1, 1)

        self.label_16 = QLabel(self.visible_group)
        self.label_16.setObjectName(u"label_16")

        self.gridLayout_3.addWidget(self.label_16, 2, 0, 1, 1)

        self.visible_min = QLabel(self.visible_group)
        self.visible_min.setObjectName(u"visible_min")
        self.visible_min.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_min, 3, 1, 1, 1)

        self.visible_std = QLabel(self.visible_group)
        self.visible_std.setObjectName(u"visible_std")
        self.visible_std.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_std, 7, 1, 1, 1)

        self.xunit4 = QLabel(self.visible_group)
        self.xunit4.setObjectName(u"xunit4")
        self.xunit4.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.xunit4, 0, 2, 1, 1)

        self.label_39 = QLabel(self.visible_group)
        self.label_39.setObjectName(u"label_39")

        self.gridLayout_3.addWidget(self.label_39, 9, 0, 1, 1)

        self.label_8 = QLabel(self.visible_group)
        self.label_8.setObjectName(u"label_8")

        self.gridLayout_3.addWidget(self.label_8, 1, 0, 1, 1)

        self.label_7 = QLabel(self.visible_group)
        self.label_7.setObjectName(u"label_7")

        self.gridLayout_3.addWidget(self.label_7, 0, 0, 1, 1)

        self.unit6 = QLabel(self.visible_group)
        self.unit6.setObjectName(u"unit6")
        self.unit6.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.unit6, 4, 2, 1, 1)

        self.xunit6 = QLabel(self.visible_group)
        self.xunit6.setObjectName(u"xunit6")
        self.xunit6.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.xunit6, 2, 2, 1, 1)

        self.visible_delta = QLabel(self.visible_group)
        self.visible_delta.setObjectName(u"visible_delta")
        self.visible_delta.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_delta, 8, 1, 1, 1)

        self.label_31 = QLabel(self.visible_group)
        self.label_31.setObjectName(u"label_31")

        self.gridLayout_3.addWidget(self.label_31, 6, 0, 1, 1)

        self.unit12 = QLabel(self.visible_group)
        self.unit12.setObjectName(u"unit12")
        self.unit12.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.unit12, 5, 2, 1, 1)

        self.label_30 = QLabel(self.visible_group)
        self.label_30.setObjectName(u"label_30")

        self.gridLayout_3.addWidget(self.label_30, 5, 0, 1, 1)

        self.visible_max = QLabel(self.visible_group)
        self.visible_max.setObjectName(u"visible_max")
        self.visible_max.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_max, 4, 1, 1, 1)

        self.visible_delta_t = QLabel(self.visible_group)
        self.visible_delta_t.setObjectName(u"visible_delta_t")
        self.visible_delta_t.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_delta_t, 2, 1, 1, 1)

        self.visible_average = QLabel(self.visible_group)
        self.visible_average.setObjectName(u"visible_average")
        self.visible_average.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_average, 5, 1, 1, 1)

        self.xunit5 = QLabel(self.visible_group)
        self.xunit5.setObjectName(u"xunit5")
        self.xunit5.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.xunit5, 1, 2, 1, 1)

        self.unit13 = QLabel(self.visible_group)
        self.unit13.setObjectName(u"unit13")
        self.unit13.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.unit13, 6, 2, 1, 1)

        self.label_40 = QLabel(self.visible_group)
        self.label_40.setObjectName(u"label_40")

        self.gridLayout_3.addWidget(self.label_40, 10, 0, 1, 1)

        self.visible_gradient_unit = QLabel(self.visible_group)
        self.visible_gradient_unit.setObjectName(u"visible_gradient_unit")
        self.visible_gradient_unit.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_gradient_unit, 9, 2, 1, 1)

        self.visible_integral_unit = QLabel(self.visible_group)
        self.visible_integral_unit.setObjectName(u"visible_integral_unit")
        self.visible_integral_unit.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_integral_unit, 10, 2, 1, 1)

        self.visible_gradient = QLabel(self.visible_group)
        self.visible_gradient.setObjectName(u"visible_gradient")
        self.visible_gradient.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_gradient, 9, 1, 1, 1)

        self.visible_integral = QLabel(self.visible_group)
        self.visible_integral.setObjectName(u"visible_integral")
        self.visible_integral.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.visible_integral, 10, 1, 1, 1)

        self.gridLayout_3.setColumnStretch(1, 1)

        self.gridLayout.addWidget(self.visible_group, 4, 0, 1, 2)

        self.overall_group = QGroupBox(self.scrollAreaWidgetContents)
        self.overall_group.setObjectName(u"overall_group")
        self.gridLayout_2 = QGridLayout(self.overall_group)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setHorizontalSpacing(1)
        self.gridLayout_2.setVerticalSpacing(1)
        self.gridLayout_2.setContentsMargins(1, 1, 1, 1)
        self.xunit7 = QLabel(self.overall_group)
        self.xunit7.setObjectName(u"xunit7")
        self.xunit7.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.xunit7, 0, 2, 1, 1)

        self.overall_delta = QLabel(self.overall_group)
        self.overall_delta.setObjectName(u"overall_delta")
        self.overall_delta.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_delta, 9, 1, 1, 1)

        self.overall_max = QLabel(self.overall_group)
        self.overall_max.setObjectName(u"overall_max")
        self.overall_max.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_max, 5, 1, 1, 1)

        self.overall_stop = QLabel(self.overall_group)
        self.overall_stop.setObjectName(u"overall_stop")
        self.overall_stop.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_stop, 1, 1, 1, 1)

        self.label_43 = QLabel(self.overall_group)
        self.label_43.setObjectName(u"label_43")

        self.gridLayout_2.addWidget(self.label_43, 9, 0, 1, 1)

        self.label_6 = QLabel(self.overall_group)
        self.label_6.setObjectName(u"label_6")

        self.gridLayout_2.addWidget(self.label_6, 5, 0, 1, 1)

        self.overall_std = QLabel(self.overall_group)
        self.overall_std.setObjectName(u"overall_std")
        self.overall_std.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_std, 8, 1, 1, 1)

        self.label_3 = QLabel(self.overall_group)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout_2.addWidget(self.label_3, 0, 0, 1, 1)

        self.label_32 = QLabel(self.overall_group)
        self.label_32.setObjectName(u"label_32")

        self.gridLayout_2.addWidget(self.label_32, 6, 0, 1, 1)

        self.unit9 = QLabel(self.overall_group)
        self.unit9.setObjectName(u"unit9")
        self.unit9.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.unit9, 5, 2, 1, 1)

        self.label_41 = QLabel(self.overall_group)
        self.label_41.setObjectName(u"label_41")

        self.gridLayout_2.addWidget(self.label_41, 10, 0, 1, 1)

        self.overall_rms = QLabel(self.overall_group)
        self.overall_rms.setObjectName(u"overall_rms")
        self.overall_rms.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_rms, 7, 1, 1, 1)

        self.xunit8 = QLabel(self.overall_group)
        self.xunit8.setObjectName(u"xunit8")
        self.xunit8.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.xunit8, 1, 2, 1, 1)

        self.overall_average = QLabel(self.overall_group)
        self.overall_average.setObjectName(u"overall_average")
        self.overall_average.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_average, 6, 1, 1, 1)

        self.unit8 = QLabel(self.overall_group)
        self.unit8.setObjectName(u"unit8")
        self.unit8.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.unit8, 4, 2, 1, 1)

        self.label_4 = QLabel(self.overall_group)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_2.addWidget(self.label_4, 1, 0, 1, 1)

        self.label_5 = QLabel(self.overall_group)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout_2.addWidget(self.label_5, 4, 0, 1, 1)

        self.unit18 = QLabel(self.overall_group)
        self.unit18.setObjectName(u"unit18")
        self.unit18.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.unit18, 8, 2, 1, 1)

        self.overall_integral_unit = QLabel(self.overall_group)
        self.overall_integral_unit.setObjectName(u"overall_integral_unit")
        self.overall_integral_unit.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_integral_unit, 11, 2, 1, 1)

        self.unit19 = QLabel(self.overall_group)
        self.unit19.setObjectName(u"unit19")
        self.unit19.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.unit19, 9, 2, 1, 1)

        self.overall_gradient_unit = QLabel(self.overall_group)
        self.overall_gradient_unit.setObjectName(u"overall_gradient_unit")
        self.overall_gradient_unit.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_gradient_unit, 10, 2, 1, 1)

        self.unit14 = QLabel(self.overall_group)
        self.unit14.setObjectName(u"unit14")
        self.unit14.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.unit14, 6, 2, 1, 1)

        self.label_33 = QLabel(self.overall_group)
        self.label_33.setObjectName(u"label_33")

        self.gridLayout_2.addWidget(self.label_33, 7, 0, 1, 1)

        self.unit15 = QLabel(self.overall_group)
        self.unit15.setObjectName(u"unit15")
        self.unit15.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.unit15, 7, 2, 1, 1)

        self.label_36 = QLabel(self.overall_group)
        self.label_36.setObjectName(u"label_36")

        self.gridLayout_2.addWidget(self.label_36, 8, 0, 1, 1)

        self.label_42 = QLabel(self.overall_group)
        self.label_42.setObjectName(u"label_42")

        self.gridLayout_2.addWidget(self.label_42, 11, 0, 1, 1)

        self.overall_start = QLabel(self.overall_group)
        self.overall_start.setObjectName(u"overall_start")
        self.overall_start.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_start, 0, 1, 1, 1)

        self.overall_min = QLabel(self.overall_group)
        self.overall_min.setObjectName(u"overall_min")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.overall_min.sizePolicy().hasHeightForWidth())
        self.overall_min.setSizePolicy(sizePolicy)
        self.overall_min.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_min, 4, 1, 1, 1)

        self.overall_integral = QLabel(self.overall_group)
        self.overall_integral.setObjectName(u"overall_integral")
        self.overall_integral.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_integral, 11, 1, 1, 1)

        self.overall_gradient = QLabel(self.overall_group)
        self.overall_gradient.setObjectName(u"overall_gradient")
        self.overall_gradient.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_gradient, 10, 1, 1, 1)

        self.label_44 = QLabel(self.overall_group)
        self.label_44.setObjectName(u"label_44")

        self.gridLayout_2.addWidget(self.label_44, 3, 0, 1, 1)

        self.overall_delta_t = QLabel(self.overall_group)
        self.overall_delta_t.setObjectName(u"overall_delta_t")
        self.overall_delta_t.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.overall_delta_t, 3, 1, 1, 1)

        self.xunit9 = QLabel(self.overall_group)
        self.xunit9.setObjectName(u"xunit9")
        self.xunit9.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.xunit9, 3, 2, 1, 1)

        self.gridLayout_2.setColumnStretch(1, 1)

        self.gridLayout.addWidget(self.overall_group, 5, 0, 1, 2)

        self.cursor_group = QGroupBox(self.scrollAreaWidgetContents)
        self.cursor_group.setObjectName(u"cursor_group")
        self.cursor_group.setFlat(False)
        self.cursor_group.setCheckable(False)
        self.gridLayout_4 = QGridLayout(self.cursor_group)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.gridLayout_4.setHorizontalSpacing(1)
        self.gridLayout_4.setVerticalSpacing(1)
        self.gridLayout_4.setContentsMargins(1, 1, 1, 1)
        self.cursor_t = QLabel(self.cursor_group)
        self.cursor_t.setObjectName(u"cursor_t")
        self.cursor_t.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_4.addWidget(self.cursor_t, 0, 1, 1, 1)

        self.label_12 = QLabel(self.cursor_group)
        self.label_12.setObjectName(u"label_12")

        self.gridLayout_4.addWidget(self.label_12, 0, 0, 1, 1)

        self.cursor_value = QLabel(self.cursor_group)
        self.cursor_value.setObjectName(u"cursor_value")
        self.cursor_value.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_4.addWidget(self.cursor_value, 1, 1, 1, 1)

        self.label_11 = QLabel(self.cursor_group)
        self.label_11.setObjectName(u"label_11")

        self.gridLayout_4.addWidget(self.label_11, 1, 0, 1, 1)

        self.xunit0 = QLabel(self.cursor_group)
        self.xunit0.setObjectName(u"xunit0")
        self.xunit0.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_4.addWidget(self.xunit0, 0, 2, 1, 1)

        self.unit1 = QLabel(self.cursor_group)
        self.unit1.setObjectName(u"unit1")
        self.unit1.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_4.addWidget(self.unit1, 1, 2, 1, 1)

        self.gridLayout_4.setColumnStretch(1, 1)

        self.gridLayout.addWidget(self.cursor_group, 2, 0, 1, 2)

        self.name = QTextEdit(self.scrollAreaWidgetContents)
        self.name.setObjectName(u"name")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(1)
        sizePolicy1.setHeightForWidth(self.name.sizePolicy().hasHeightForWidth())
        self.name.setSizePolicy(sizePolicy1)
        self.name.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.name.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.name.setSizeAdjustPolicy(QAbstractScrollArea.AdjustToContents)
        self.name.setReadOnly(True)

        self.gridLayout.addWidget(self.name, 0, 0, 1, 2)

        self.region_group = QGroupBox(self.scrollAreaWidgetContents)
        self.region_group.setObjectName(u"region_group")
        self.gridLayout_5 = QGridLayout(self.region_group)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.gridLayout_5.setHorizontalSpacing(1)
        self.gridLayout_5.setVerticalSpacing(1)
        self.gridLayout_5.setContentsMargins(1, 1, 1, 1)
        self.label_13 = QLabel(self.region_group)
        self.label_13.setObjectName(u"label_13")

        self.gridLayout_5.addWidget(self.label_13, 0, 0, 1, 1)

        self.selected_integral_unit = QLabel(self.region_group)
        self.selected_integral_unit.setObjectName(u"selected_integral_unit")
        self.selected_integral_unit.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_integral_unit, 13, 2, 1, 1)

        self.selected_rms = QLabel(self.region_group)
        self.selected_rms.setObjectName(u"selected_rms")
        self.selected_rms.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_rms, 9, 1, 1, 1)

        self.label_29 = QLabel(self.region_group)
        self.label_29.setObjectName(u"label_29")

        self.gridLayout_5.addWidget(self.label_29, 9, 0, 1, 1)

        self.selected_gradient_unit = QLabel(self.region_group)
        self.selected_gradient_unit.setObjectName(u"selected_gradient_unit")
        self.selected_gradient_unit.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_gradient_unit, 12, 2, 1, 1)

        self.selected_start = QLabel(self.region_group)
        self.selected_start.setObjectName(u"selected_start")
        self.selected_start.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_start, 0, 1, 1, 1)

        self.label_15 = QLabel(self.region_group)
        self.label_15.setObjectName(u"label_15")

        self.gridLayout_5.addWidget(self.label_15, 1, 0, 1, 1)

        self.selected_delta_t = QLabel(self.region_group)
        self.selected_delta_t.setObjectName(u"selected_delta_t")
        self.selected_delta_t.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_delta_t, 3, 1, 1, 1)

        self.label_2 = QLabel(self.region_group)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_5.addWidget(self.label_2, 4, 0, 1, 1)

        self.label_14 = QLabel(self.region_group)
        self.label_14.setObjectName(u"label_14")

        self.gridLayout_5.addWidget(self.label_14, 6, 0, 1, 1)

        self.label = QLabel(self.region_group)
        self.label.setObjectName(u"label")

        self.gridLayout_5.addWidget(self.label, 8, 0, 1, 1)

        self.selected_stop = QLabel(self.region_group)
        self.selected_stop.setObjectName(u"selected_stop")
        self.selected_stop.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_stop, 1, 1, 1, 1)

        self.unit16 = QLabel(self.region_group)
        self.unit16.setObjectName(u"unit16")
        self.unit16.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit16, 10, 2, 1, 1)

        self.label_34 = QLabel(self.region_group)
        self.label_34.setObjectName(u"label_34")

        self.gridLayout_5.addWidget(self.label_34, 10, 0, 1, 1)

        self.xunit1 = QLabel(self.region_group)
        self.xunit1.setObjectName(u"xunit1")
        self.xunit1.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.xunit1, 0, 2, 1, 1)

        self.xunit3 = QLabel(self.region_group)
        self.xunit3.setObjectName(u"xunit3")
        self.xunit3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.xunit3, 3, 2, 1, 1)

        self.label_21 = QLabel(self.region_group)
        self.label_21.setObjectName(u"label_21")

        self.gridLayout_5.addWidget(self.label_21, 3, 0, 1, 1)

        self.selected_max = QLabel(self.region_group)
        self.selected_max.setObjectName(u"selected_max")
        self.selected_max.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_max, 7, 1, 1, 1)

        self.label_17 = QLabel(self.region_group)
        self.label_17.setObjectName(u"label_17")

        self.gridLayout_5.addWidget(self.label_17, 7, 0, 1, 1)

        self.label_37 = QLabel(self.region_group)
        self.label_37.setObjectName(u"label_37")

        self.gridLayout_5.addWidget(self.label_37, 12, 0, 1, 1)

        self.selected_std = QLabel(self.region_group)
        self.selected_std.setObjectName(u"selected_std")
        self.selected_std.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_std, 10, 1, 1, 1)

        self.label_18 = QLabel(self.region_group)
        self.label_18.setObjectName(u"label_18")

        self.gridLayout_5.addWidget(self.label_18, 11, 0, 1, 1)

        self.xunit2 = QLabel(self.region_group)
        self.xunit2.setObjectName(u"xunit2")
        self.xunit2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.xunit2, 1, 2, 1, 1)

        self.unit11 = QLabel(self.region_group)
        self.unit11.setObjectName(u"unit11")
        self.unit11.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit11, 9, 2, 1, 1)

        self.selected_min = QLabel(self.region_group)
        self.selected_min.setObjectName(u"selected_min")
        self.selected_min.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_min, 6, 1, 1, 1)

        self.label_38 = QLabel(self.region_group)
        self.label_38.setObjectName(u"label_38")

        self.gridLayout_5.addWidget(self.label_38, 13, 0, 1, 1)

        self.unit3 = QLabel(self.region_group)
        self.unit3.setObjectName(u"unit3")
        self.unit3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit3, 7, 2, 1, 1)

        self.selected_delta = QLabel(self.region_group)
        self.selected_delta.setObjectName(u"selected_delta")
        self.selected_delta.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_delta, 11, 1, 1, 1)

        self.selected_integral = QLabel(self.region_group)
        self.selected_integral.setObjectName(u"selected_integral")
        self.selected_integral.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_integral, 13, 1, 1, 1)

        self.unit4 = QLabel(self.region_group)
        self.unit4.setObjectName(u"unit4")
        self.unit4.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit4, 11, 2, 1, 1)

        self.selected_average = QLabel(self.region_group)
        self.selected_average.setObjectName(u"selected_average")
        self.selected_average.setAutoFillBackground(False)
        self.selected_average.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_average, 8, 1, 1, 1)

        self.selected_gradient = QLabel(self.region_group)
        self.selected_gradient.setObjectName(u"selected_gradient")
        self.selected_gradient.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_gradient, 12, 1, 1, 1)

        self.unit10 = QLabel(self.region_group)
        self.unit10.setObjectName(u"unit10")
        self.unit10.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit10, 8, 2, 1, 1)

        self.unit2 = QLabel(self.region_group)
        self.unit2.setObjectName(u"unit2")
        self.unit2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit2, 6, 2, 1, 1)

        self.label_20 = QLabel(self.region_group)
        self.label_20.setObjectName(u"label_20")

        self.gridLayout_5.addWidget(self.label_20, 5, 0, 1, 1)

        self.selected_left = QLabel(self.region_group)
        self.selected_left.setObjectName(u"selected_left")
        self.selected_left.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_left, 4, 1, 1, 1)

        self.selected_right = QLabel(self.region_group)
        self.selected_right.setObjectName(u"selected_right")
        self.selected_right.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.selected_right, 5, 1, 1, 1)

        self.unit21 = QLabel(self.region_group)
        self.unit21.setObjectName(u"unit21")
        self.unit21.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit21, 4, 2, 1, 1)

        self.unit22 = QLabel(self.region_group)
        self.unit22.setObjectName(u"unit22")
        self.unit22.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_5.addWidget(self.unit22, 5, 2, 1, 1)

        self.gridLayout_5.setColumnStretch(1, 1)

        self.gridLayout.addWidget(self.region_group, 3, 0, 1, 2)

        self.precision = QComboBox(self.scrollAreaWidgetContents)
        self.precision.setObjectName(u"precision")

        self.gridLayout.addWidget(self.precision, 1, 1, 1, 1)

        self.label_22 = QLabel(self.scrollAreaWidgetContents)
        self.label_22.setObjectName(u"label_22")

        self.gridLayout.addWidget(self.label_22, 1, 0, 1, 1)

        self.gridLayout.setRowStretch(0, 1)
        self.gridLayout.setColumnStretch(1, 1)

        self.verticalLayout.addLayout(self.gridLayout)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.verticalLayout_2.addWidget(self.scrollArea)


        self.retranslateUi(ChannelStats)

        QMetaObject.connectSlotsByName(ChannelStats)
    # setupUi

    def retranslateUi(self, ChannelStats):
        ChannelStats.setWindowTitle(QCoreApplication.translate("ChannelStats", u"Form", None))
        self.visible_group.setTitle(QCoreApplication.translate("ChannelStats", u"Visible region", None))
        self.visible_start.setText("")
        self.unit7.setText("")
        self.label_10.setText(QCoreApplication.translate("ChannelStats", u"Max", None))
        self.label_35.setText(QCoreApplication.translate("ChannelStats", u"STD", None))
        self.unit5.setText("")
        self.visible_stop.setText("")
        self.label_19.setText(QCoreApplication.translate("ChannelStats", u"\u0394", None))
        self.label_9.setText(QCoreApplication.translate("ChannelStats", u"Min", None))
        self.visible_rms.setText("")
        self.unit17.setText("")
        self.label_16.setText(QCoreApplication.translate("ChannelStats", u"\u0394t", None))
        self.visible_min.setText("")
        self.visible_std.setText("")
        self.xunit4.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.label_39.setText(QCoreApplication.translate("ChannelStats", u"Gradient", None))
        self.label_8.setText(QCoreApplication.translate("ChannelStats", u"t2", None))
        self.label_7.setText(QCoreApplication.translate("ChannelStats", u"t1", None))
        self.unit6.setText("")
        self.xunit6.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.visible_delta.setText("")
        self.label_31.setText(QCoreApplication.translate("ChannelStats", u"RMS", None))
        self.unit12.setText("")
        self.label_30.setText(QCoreApplication.translate("ChannelStats", u"Avg", None))
        self.visible_max.setText("")
        self.visible_delta_t.setText("")
        self.visible_average.setText("")
        self.xunit5.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.unit13.setText("")
        self.label_40.setText(QCoreApplication.translate("ChannelStats", u"Integral", None))
        self.visible_gradient_unit.setText("")
        self.visible_integral_unit.setText("")
        self.visible_gradient.setText("")
        self.visible_integral.setText("")
        self.overall_group.setTitle(QCoreApplication.translate("ChannelStats", u"Overall", None))
        self.xunit7.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.overall_delta.setText("")
        self.overall_max.setText("")
        self.overall_stop.setText("")
        self.label_43.setText(QCoreApplication.translate("ChannelStats", u"\u0394", None))
        self.label_6.setText(QCoreApplication.translate("ChannelStats", u"Max", None))
        self.overall_std.setText("")
        self.label_3.setText(QCoreApplication.translate("ChannelStats", u"t1", None))
        self.label_32.setText(QCoreApplication.translate("ChannelStats", u"Avg", None))
        self.unit9.setText("")
        self.label_41.setText(QCoreApplication.translate("ChannelStats", u"Grad", None))
        self.overall_rms.setText("")
        self.xunit8.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.overall_average.setText("")
        self.unit8.setText("")
        self.label_4.setText(QCoreApplication.translate("ChannelStats", u"t2", None))
        self.label_5.setText(QCoreApplication.translate("ChannelStats", u"Min", None))
        self.unit18.setText("")
        self.overall_integral_unit.setText("")
        self.unit19.setText("")
        self.overall_gradient_unit.setText("")
        self.unit14.setText("")
        self.label_33.setText(QCoreApplication.translate("ChannelStats", u"RMS", None))
        self.unit15.setText("")
        self.label_36.setText(QCoreApplication.translate("ChannelStats", u"STD", None))
        self.label_42.setText(QCoreApplication.translate("ChannelStats", u"Integral", None))
        self.overall_start.setText("")
        self.overall_min.setText("")
        self.overall_integral.setText("")
        self.overall_gradient.setText("")
        self.label_44.setText(QCoreApplication.translate("ChannelStats", u"\u0394t", None))
        self.overall_delta_t.setText("")
        self.xunit9.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.cursor_group.setTitle(QCoreApplication.translate("ChannelStats", u"Cursor", None))
        self.cursor_t.setText("")
        self.label_12.setText(QCoreApplication.translate("ChannelStats", u"Time", None))
        self.cursor_value.setText("")
        self.label_11.setText(QCoreApplication.translate("ChannelStats", u"Value", None))
        self.xunit0.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.unit1.setText("")
        self.name.setHtml(QCoreApplication.translate("ChannelStats", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"</style></head><body style=\" font-family:'Roboto Condensed'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Consolas';\"><br /></p></body></html>", None))
        self.region_group.setTitle(QCoreApplication.translate("ChannelStats", u"Selected region", None))
        self.label_13.setText(QCoreApplication.translate("ChannelStats", u"t1", None))
        self.selected_integral_unit.setText("")
        self.selected_rms.setText("")
        self.label_29.setText(QCoreApplication.translate("ChannelStats", u"RMS", None))
        self.selected_gradient_unit.setText("")
        self.selected_start.setText("")
        self.label_15.setText(QCoreApplication.translate("ChannelStats", u"t2", None))
        self.selected_delta_t.setText("")
        self.label_2.setText(QCoreApplication.translate("ChannelStats", u"First", None))
        self.label_14.setText(QCoreApplication.translate("ChannelStats", u"Min", None))
        self.label.setText(QCoreApplication.translate("ChannelStats", u"Avg", None))
        self.selected_stop.setText("")
        self.unit16.setText("")
        self.label_34.setText(QCoreApplication.translate("ChannelStats", u"STD", None))
        self.xunit1.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.xunit3.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.label_21.setText(QCoreApplication.translate("ChannelStats", u"\u0394t", None))
        self.selected_max.setText("")
        self.label_17.setText(QCoreApplication.translate("ChannelStats", u"Max", None))
        self.label_37.setText(QCoreApplication.translate("ChannelStats", u"Gradient", None))
        self.selected_std.setText("")
        self.label_18.setText(QCoreApplication.translate("ChannelStats", u"\u0394", None))
        self.xunit2.setText(QCoreApplication.translate("ChannelStats", u" s", None))
        self.unit11.setText("")
        self.selected_min.setText("")
        self.label_38.setText(QCoreApplication.translate("ChannelStats", u"Integral", None))
        self.unit3.setText("")
        self.selected_delta.setText("")
        self.selected_integral.setText("")
        self.unit4.setText("")
        self.selected_average.setText("")
        self.selected_gradient.setText("")
        self.unit10.setText("")
        self.unit2.setText("")
        self.label_20.setText(QCoreApplication.translate("ChannelStats", u"Last", None))
        self.selected_left.setText("")
        self.selected_right.setText("")
        self.unit21.setText("")
        self.unit22.setText("")
        self.label_22.setText(QCoreApplication.translate("ChannelStats", u"Precisison", None))
    # retranslateUi



from PySide6 import QtCore, QtTest

from asammdf.gui.dialogs.window_selection_dialog import WindowSelectionDialog
from test.asammdf.gui.test_base import TestBase


class TestWindowSelectionDialog(TestBase):
    def test_RadioButtons(self):
        """
        Test Scope:
            - Ensure that correct radio button is identified
        Events:
            - Check 1st Radio Button
            - Press Apply
            - Check 2nd Radio Button
            - Press Cancel
            - Check 3rd Radio Button
        Evaluate:
            - Evaluate that selected type correspond to RadioButton Checked
        """
        options = ("Plot", "Numeric", "Tabular")
        self.ws = WindowSelectionDialog(options=options)

        for index, option in enumerate(options):
            with self.subTest(f"{self.id()}_{option}"):
                self.ws.showNormal()

                rb = self.ws.selection_layout.itemAt(index).widget()
                self.mouseClick_RadioButton(rb)

                ok, cancel = self.ws.buttonBox.buttons()
                QtTest.QTest.mouseClick(ok, QtCore.Qt.MouseButton.LeftButton)

                self.assertEqual(option, self.ws.selected_type())

    def test_CheckBoxButton(self):
        """
        Purpose:
            - Ensure that state of checkbox is stored.
        Event:
            - Mark CheckBox Button as checked.
            - Mark CheckBox Button as unchecked.
        Evaluate:
            - Evaluate checkbox state
        """
        self.ws = WindowSelectionDialog()

        with self.subTest(f"{self.id()}_Checked"):
            self.ws.showNormal()

            self.mouseClick_CheckboxButton(self.ws.disable_channels)
            ok, cancel = self.ws.buttonBox.buttons()
            QtTest.QTest.mouseClick(ok, QtCore.Qt.MouseButton.LeftButton)

            self.assertTrue(self.ws.disable_new_channels())

        with self.subTest(f"{self.id()}_UnChecked"):
            self.ws.showNormal()

            self.mouseClick_CheckboxButton(self.ws.disable_channels)
            ok, cancel = self.ws.buttonBox.buttons()
            QtTest.QTest.mouseClick(ok, QtCore.Qt.MouseButton.LeftButton)

            self.assertFalse(self.ws.disable_new_channels())



#!/usr/bin/env python
import pathlib
import shutil
import time
from unittest import mock

from PySide6 import QtCore, QtTest, QtWidgets

from test.asammdf.gui.test_base import OpenMDF
from test.asammdf.gui.widgets.test_BaseFileWidget import TestFileWidget

# Note: If it's possible and make sense, use self.subTests
# to avoid initializing widgets multiple times and consume time.


class TestTabModifyAndExport(TestFileWidget):
    def test_PushButton_ScrambleTexts(self):
        """
        Events:
            - Open 'FileWidget' with valid measurement.
            - Go to Tab: "Modify & Export": Index 1
            - Press PushButton "Scramble texts"
        Evaluate:
            - New file is created
            - No channel from first file is found in 2nd file (scrambled file)
        """
        # Setup
        measurement_file = str(pathlib.Path(self.test_workspace, "ASAP2_Demo_V171.mf4"))
        shutil.copy(pathlib.Path(self.resource, "ASAP2_Demo_V171.mf4"), measurement_file)
        # Event
        self.setUpFileWidget(measurement_file=measurement_file, default=True)
        # Go to Tab: "Modify & Export": Index 1
        self.widget.aspects.setCurrentIndex(1)
        # Press PushButton ScrambleTexts
        QtTest.QTest.mouseClick(self.widget.scramble_btn, QtCore.Qt.MouseButton.LeftButton)

        channels = list(self.widget.mdf.channels_db)

        # Evaluate
        scrambled_filepath = pathlib.Path(self.test_workspace, "ASAP2_Demo_V171.scrambled.mf4")
        # Wait for Thread to finish
        time.sleep(0.1)

        with OpenMDF(scrambled_filepath) as mdf_file:
            result = filter(lambda c: c in mdf_file.channels_db, channels)
            self.assertFalse(any(result))

    def test_ExportMDF(self):
        """
        When QThreads are running, event-loops needs to be processed.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Go to Tab: "Modify & Export": Index 1
            - Set "channel_view" to "Natural sort"
            - Select two channels
            - Ensure that output format is MDF
            - Case 0:
                - Press PushButton Apply.
                    - Simulate that no valid path is provided.
            - Case 1:
                - Press PushButton Apply.
                    - Simulate that no valid path is provided.
        Evaluate:
            - Evaluate that file was created.
            - Open File and check that there are only two channels.
        """
        # Setup
        measurement_file = str(pathlib.Path(self.resource, "ASAP2_Demo_V171.mf4"))
        # Event
        self.setUpFileWidget(measurement_file=measurement_file, default=True)
        # Go to Tab: "Modify & Export": Index 1
        self.widget.aspects.setCurrentIndex(1)
        self.widget.filter_view.setCurrentText("Natural sort")

        count = 2
        selected_channels = []
        iterator = QtWidgets.QTreeWidgetItemIterator(self.widget.filter_tree)
        while iterator.value() and count:
            item = iterator.value()
            item.setCheckState(0, QtCore.Qt.CheckState.Checked)
            self.assertTrue(item.checkState(0) == QtCore.Qt.CheckState.Checked)
            selected_channels.append(item.text(0))
            iterator += 1
            count -= 1
        # Evaluate that channels were added to "selected_filter_channels"
        for index in range(self.widget.selected_filter_channels.count()):
            item = self.widget.selected_filter_channels.item(index)
            self.assertIn(item.text(), selected_channels)

        self.widget.output_format.setCurrentText("MDF")

        # Case 0:
        self.processEvents()
        with self.subTest("test_ExportMDF_0"):
            with (
                mock.patch("asammdf.gui.widgets.file.QtWidgets.QFileDialog.getSaveFileName") as mc_getSaveFileName,
                mock.patch("asammdf.gui.widgets.file.setup_progress") as mo_setup_progress,
            ):
                mc_getSaveFileName.return_value = None, None
                QtTest.QTest.mouseClick(self.widget.apply_btn, QtCore.Qt.MouseButton.LeftButton)
                self.processEvents()
            # Evaluate
            # Progress is not created
            mo_setup_progress.assert_not_called()

        # Case 1:
        self.processEvents()
        with self.subTest("test_ExportMDF_1"):
            saved_file = pathlib.Path(self.test_workspace, f"{self.id()}.mf4")
            with mock.patch("asammdf.gui.widgets.file.QtWidgets.QFileDialog.getSaveFileName") as mc_getSaveFileName:
                mc_getSaveFileName.return_value = str(saved_file), None
                QtTest.QTest.mouseClick(self.widget.apply_btn, QtCore.Qt.MouseButton.LeftButton)
                self.processEvents(1)
        # Wait for thread to finish
        self.processEvents(1)

        # Evaluate
        self.assertTrue(saved_file.exists())

        # TearDown Widget
        with OpenMDF(saved_file) as mdf_file:
            selected_channels.append("time")
            self.assertListEqual(sorted(selected_channels), sorted(mdf_file.channels_db))



from datetime import datetime, timedelta, timezone
from math import ceil
from traceback import format_exc

import numpy as np
import pandas as pd
import pyqtgraph as pg
import pyqtgraph.functions as fn
from pyqtgraph.graphicsItems.ButtonItem import ButtonItem
from pyqtgraph.Point import Point
from PySide6 import QtCore, QtGui, QtWidgets

LOCAL_TIMEZONE = datetime.now(timezone.utc).astimezone().tzinfo
BUTTON_SIZE = 16


class FormatedAxis(pg.AxisItem):
    rangeChanged = QtCore.Signal(object, object)
    scale_editor_requested = QtCore.Signal(object)

    def __init__(self, *args, **kwargs):
        self.plus = self.minus = None
        self.uuid = kwargs.pop("uuid", None)
        self.background = kwargs.pop("background", fn.mkColor("#000000"))
        self.linked_signal = kwargs.pop("linked_signal", None)

        super().__init__(*args, **kwargs)

        self.setAcceptHoverEvents(True)

        self._settings = QtCore.QSettings()

        self.format = "phys"
        self.mode = "phys"
        self.text_conversion = None
        self.origin = None

        self.hovering = False

        self._label_with_unit = ""

        self.locked = kwargs.pop("locked", False)

        if self.orientation in ("left", "right"):
            self.plus = ButtonItem(":/plus.png", BUTTON_SIZE, parentItem=self)
            self.minus = ButtonItem(":/minus.png", BUTTON_SIZE, parentItem=self)

            self.plus.clicked.connect(self.increase_width)
            self.minus.clicked.connect(self.decrease_width)

            if self.scene() is not None:
                self.scene().addItem(self.plus)
                self.scene().addItem(self.minus)

            self.setWidth(48)

        self.set_pen(self._pen)

    def increase_width(self):
        self.setWidth(self.width() + 10)

    def decrease_width(self):
        self.setWidth(self.width() - 10)

    def setWidth(self, w=None):
        if self.orientation in ("left", "right"):
            super().setWidth(max(w, 48))
        else:
            super().setWidth(w)

    def tickStrings(self, values, scale, spacing):
        strns = []
        self.tick_positions = values

        if self.text_conversion and self.mode == "phys":
            strns = []
            for val in values:
                nv = self.text_conversion.convert(np.array([val]), as_bytes=True)[0]

                val = float(val)

                if val.is_integer():
                    val = int(val)

                    if self.format == "hex":
                        val = hex(int(val))
                    elif self.format == "bin":
                        val = bin(int(val))
                    else:
                        val = str(val)
                else:
                    val = f"{val:.6f}"

                if isinstance(nv, bytes):
                    try:
                        strns.append(f'{val}={nv.decode("utf-8", errors="replace")}')
                    except:
                        strns.append(f'{val}={nv.decode("latin-1", errors="replace")}')
                else:
                    strns.append(val)
        else:
            if self.format == "phys":
                strns = super().tickStrings(values, scale, spacing)

            elif self.format == "hex":
                for val in values:
                    val = float(val)
                    if val.is_integer():
                        val = hex(int(val))
                    else:
                        val = ""
                    strns.append(val)

            elif self.format == "bin":
                for val in values:
                    val = float(val)
                    if val.is_integer():
                        val = bin(int(val))
                    else:
                        val = ""
                    strns.append(val)
            elif self.format == "ascii":
                for val in values:
                    val = float(val)
                    if val.is_integer():
                        val = int(val)
                        if 0 < val < 0x110000:
                            val = chr(val)
                        else:
                            val = str(val)
                    else:
                        val = ""
                    strns.append(val)
            elif self.format == "time":
                strns = [str(timedelta(seconds=val)) for val in values]
            elif self.format == "date":
                strns = (
                    (
                        pd.to_timedelta(
                            np.array(values),
                            unit="s",
                            errors="coerce",
                        )
                        + self.origin
                    )
                    .astype(str)
                    .to_list()
                )

        return [val[:80] for val in strns]

    def setLabel(self, text=None, units=None, unitPrefix=None, **args):
        """overwrites pyqtgraph setLabel"""
        show_label = False
        if text is not None:
            self.labelText = text
            show_label = True
        if units is not None:
            self.labelUnits = units
            show_label = True
        if show_label:
            self.showLabel()
        if unitPrefix is not None:
            self.labelUnitPrefix = unitPrefix
        if len(args) > 0:
            self.labelStyle = args
        self.label.setHtml(self.labelString())
        self._adjustSize()
        self.picture = None
        self.update()

    def labelString(self):
        if self.labelUnits == "":
            if not self.autoSIPrefix or self.autoSIPrefixScale == 1.0:
                units = ""
            else:
                units = "(x%g)" % (1.0 / self.autoSIPrefixScale)
        else:
            units = f"({self.labelUnitPrefix}{self.labelUnits})"

        s = f"{self.labelText} {units}"
        self._label_with_unit = s

        style = ";".join([f"{k}: {self.labelStyle[k]}" for k in self.labelStyle])

        lsbl = f"<span style='{style}'>{s}</span>"
        return lsbl

    def mouseDragEvent(self, event):
        if self.locked:
            return

        if self.orientation in ("left", "right"):
            ev = event
            ev.accept()

            pos = ev.pos()
            lastPos = ev.lastPos()
            dif = pos - lastPos
            dif = dif * -1

            if ev.button() in [
                QtCore.Qt.MouseButton.LeftButton,
                QtCore.Qt.MouseButton.MiddleButton,
            ]:
                if self.orientation in ("left", "right"):
                    scale = (self.range[1] - self.range[0]) / self.sceneBoundingRect().height()
                    delta = scale * dif.y()
                else:
                    scale = (self.range[1] - self.range[0]) / self.sceneBoundingRect().width()
                    delta = scale * dif.x()

                self.setRange(self.range[0] - delta, self.range[1] - delta)

            elif ev.button() & QtCore.Qt.MouseButton.RightButton:
                mid = sum(self.range) / 2
                delta = self.range[-1] - self.range[0]

                if self.orientation in ("left", "right"):
                    if dif.y() > 0:
                        delta = 0.94 * delta
                    else:
                        delta = 1.06 * delta

                else:
                    if dif.x() > 0:
                        delta = 0.94 * delta
                    else:
                        delta = 1.06 * delta

                self.setRange(mid - delta / 2, mid + delta / 2)
        else:
            return self.linkedView().mouseDragEvent(event, axis=0, ignore_cursor=True)

    def mouseClickEvent(self, event):
        if event.button() == QtCore.Qt.MouseButton.RightButton:
            if self.sceneBoundingRect().contains(event.scenePos()):
                event.accept()
                self.raiseContextMenu(event)
        else:
            lv = self.linkedView()
            if lv is None:
                return
            return lv.mouseClickEvent(event)

    def hoverEnterEvent(self, event):
        self.hovering = True
        self.picture = None
        self.update()
        super().hoverEnterEvent(event)

    def hoverLeaveEvent(self, event):
        self.hovering = False
        self.picture = None
        self.update()
        super().hoverLeaveEvent(event)

    def resizeEvent(self, ev=None):
        if self.orientation in ("left", "right"):
            nudge = 5

            if self.minus is not None:
                br = self.minus.boundingRect()
                p = QtCore.QPointF(0, 0)
                if self.orientation == "left":
                    p.setY(5)
                    p.setX(nudge)
                elif self.orientation == "right":
                    p.setY(5)
                    p.setX(int(self.size().width() - br.height() + nudge))

                self.minus.setPos(p)

            if self.plus is not None:
                br = self.plus.boundingRect()
                p = QtCore.QPointF(0, 0)
                if self.orientation == "left":
                    p.setY(26)
                    p.setX(nudge)
                elif self.orientation == "right":
                    p.setY(26)
                    p.setX(int(self.size().width() - br.height() + nudge))

                self.plus.setPos(p)

        return super().resizeEvent(ev)

    def close(self):
        if self.plus is not None:
            self.scene().removeItem(self.plus)
        if self.minus is not None:
            self.scene().removeItem(self.minus)
        self.plus = None
        self.minus = None

        super().close()

    def set_pen(self, pen=None):
        if pen is None:
            pen = fn.mkPen(pen)

        color = pen.color()

        if self.minus is not None:
            p = QtGui.QPainter(self.minus.pixmap)
            self.minus.pixmap.fill(color)
            p.setBrush(self.background)
            p.drawRect(QtCore.QRect(4, 24, 56, 15))
            p.end()

        if self.plus is not None:
            p = QtGui.QPainter(self.plus.pixmap)
            self.plus.pixmap.fill(color)
            p.setBrush(self.background)
            p.drawRect(QtCore.QRect(4, 24, 56, 15))
            p.drawRect(QtCore.QRect(24, 4, 15, 56))
            p.end()

        if pen is not self._pen:
            self.setPen(pen)

    def raiseContextMenu(self, ev):
        low, high = self.range

        if self.orientation in ("left", "right"):
            if self.linked_signal is None:
                axis = f"{self.linked_signal.name} Y"
            else:
                axis = "Y"
        else:
            axis = "X"

        menu = QtWidgets.QMenu()
        menu.addAction(f"Edit {axis} axis scaling")
        menu.addSeparator()
        menu.addAction("Apply new axis limits")
        menu.addSeparator()

        widget = QtWidgets.QWidget()
        layout = QtWidgets.QHBoxLayout()
        widget.setLayout(layout)
        layout.addWidget(QtWidgets.QLabel("max:"))

        upper = QtWidgets.QDoubleSpinBox()
        upper.setDecimals(9)
        upper.setMinimum(-1e64)
        upper.setMaximum(1e35)
        upper.setValue(high)
        layout.addWidget(upper)

        a = QtWidgets.QWidgetAction(self)
        a.setDefaultWidget(widget)
        menu.addAction(a)

        widget = QtWidgets.QWidget()
        layout = QtWidgets.QHBoxLayout()
        widget.setLayout(layout)
        layout.addWidget(QtWidgets.QLabel("min:"))

        lower = QtWidgets.QDoubleSpinBox()
        lower.setDecimals(9)
        lower.setMinimum(-1e64)
        lower.setMaximum(1e35)
        lower.setValue(low)
        layout.addWidget(lower)

        a = QtWidgets.QWidgetAction(self)
        a.setDefaultWidget(widget)
        menu.addAction(a)

        action = menu.exec_(ev.screenPos().toPoint())

        if action is None:
            return

        elif action.text() == "Apply new axis limits":
            if self.orientation in ("left", "right"):
                self.setRange(lower.value(), upper.value())
            else:
                self.setRange(lower.value(), upper.value())

        elif action.text() == f"Edit {axis} axis scaling":
            self.scale_editor_requested.emit(self.uuid)

    def set_font_size(self, size):
        font = self.font()
        font.setPointSize(size)

        if self.orientation in ("top", "bottom"):
            metric = QtGui.QFontMetrics(font)
            height = max(metric.height() + 2, 18)
            self.setStyle(tickFont=font, tickTextHeight=height)
        else:
            self.setStyle(tickFont=font)
            self.setFont(font)

        self.label.setFont(font)

    def setRange(self, mn, mx):
        if mn > mx:
            mn, mx = mx, mn
        super().setRange(mn, mx)
        self.rangeChanged.emit(self.uuid, (mn, mx))

    def wheelEvent(self, event):
        if self.locked:
            return

        lv = self.linkedView()

        factor = self._settings.value("zoom_wheel_factor", 0.165, type=float)

        if lv is None:
            # this is one of the individual axis

            pos = event.pos()
            rect = self.boundingRect()

            zoom_y_mode = self._settings.value("zoom_y_mode", "")

            if not zoom_y_mode:
                if self._settings.value("zoom_y_center_on_cursor", False, type=bool):
                    zoom_y_mode = "center_on_cursor"
                else:
                    zoom_y_mode = "center_on_mouse"

            if zoom_y_mode == "pin_zero_level":
                plot, uuid = self.linked_signal

                y_pos_val, sig_y_bottom, sig_y_top = plot.value_at_cursor(uuid)
                delta_proc = sig_y_top / (sig_y_top - sig_y_bottom)

                delta = sig_y_top - sig_y_bottom

                if event.delta() > 0:
                    delta *= 1 / (1 + factor)
                else:
                    delta *= 1 + factor

                end = delta_proc * delta
                start = end - delta

                self.setRange(start, end)

            elif zoom_y_mode == "center_on_cursor":
                plot, uuid = self.linked_signal

                y_pos_val, sig_y_bottom, sig_y_top = plot.value_at_cursor(uuid)

                delta_proc = (y_pos_val - (sig_y_top + sig_y_bottom) / 2) / (sig_y_top - sig_y_bottom)
                shift = delta_proc * (sig_y_top - sig_y_bottom)
                sig_y_top, sig_y_bottom = sig_y_top + shift, sig_y_bottom + shift

                delta = sig_y_top - sig_y_bottom

                if event.delta() > 0:
                    end = sig_y_top - 1 / factor * delta
                    start = sig_y_bottom + 1 / factor * delta
                else:
                    end = sig_y_top + factor * delta
                    start = sig_y_bottom - factor * delta

                self.setRange(start, end)

            elif zoom_y_mode == "center_on_mouse":
                y_pos_val = ((rect.height() + rect.y()) - pos.y()) / rect.height() * (
                    self.range[-1] - self.range[0]
                ) + self.range[0]

                ratio = abs((pos.y() - (rect.height() + rect.y())) / rect.height())

                delta = self.range[-1] - self.range[0]

                if event.delta() > 0:
                    delta = 1 / (1 + factor) * delta
                else:
                    delta = (1 + factor) * delta

                start = y_pos_val - ratio * delta
                end = y_pos_val + (1 - ratio) * delta

                self.setRange(start, end)

            event.accept()
        else:
            # this is the main Y axis or the X axis
            if self.orientation in ("top", "bottom"):
                super().wheelEvent(event)
            else:
                # main Y axis

                if not lv.state["mouseEnabled"][1]:
                    event.ignore()
                else:
                    # the plot is not Y locked
                    pos = event.pos()
                    rect = self.boundingRect()

                    zoom_y_mode = self._settings.value("zoom_y_mode", "")

                    if not zoom_y_mode:
                        if self._settings.value("zoom_y_center_on_cursor", False, type=bool):
                            zoom_y_mode = "center_on_cursor"
                        else:
                            zoom_y_mode = "center_on_mouse"

                    if zoom_y_mode == "pin_zero_level":
                        plot, uuid = self.linked_signal

                        y_pos_val, sig_y_bottom, sig_y_top = plot.value_at_cursor()
                        delta_proc = sig_y_top / (sig_y_top - sig_y_bottom)

                        delta = sig_y_top - sig_y_bottom

                        if event.delta() > 0:
                            delta *= 1 / (1 + factor)
                        else:
                            delta *= 1 + factor

                        end = delta_proc * delta
                        start = end - delta

                        self.setRange(start, end)

                    elif zoom_y_mode == "center_on_cursor":
                        plot, uuid = self.linked_signal

                        y_pos_val, sig_y_bottom, sig_y_top = plot.value_at_cursor()

                        if isinstance(y_pos_val, (int, float)):
                            delta_proc = (y_pos_val - (sig_y_top + sig_y_bottom) / 2) / (sig_y_top - sig_y_bottom)
                            shift = delta_proc * (sig_y_top - sig_y_bottom)
                            sig_y_top, sig_y_bottom = sig_y_top + shift, sig_y_bottom + shift

                        delta = sig_y_top - sig_y_bottom

                        if event.delta() > 0:
                            end = sig_y_top - factor * delta
                            start = sig_y_bottom + factor * delta
                        else:
                            end = sig_y_top + factor * delta
                            start = sig_y_bottom - factor * delta

                        self.setRange(start, end)

                    else:
                        y_pos_val = ((rect.height() + rect.y()) - pos.y()) / rect.height() * (
                            self.range[-1] - self.range[0]
                        ) + self.range[0]

                        ratio = abs((pos.y() - (rect.height() + rect.y())) / rect.height())

                        delta = self.range[-1] - self.range[0]

                        if event.delta() > 0:
                            delta = (1 - factor) * delta
                        else:
                            delta = (1 + factor) * delta

                        start = y_pos_val - ratio * delta
                        end = y_pos_val + (1 - ratio) * delta

                        self.setRange(start, end)

                    event.accept()

    def drawPicture(self, p, axisSpec, tickSpecs, textSpecs, ratio=1.0):
        p.setRenderHint(p.RenderHint.Antialiasing, False)
        p.setRenderHint(p.RenderHint.TextAntialiasing, True)

        bounding = self.boundingRect().toAlignedRect()
        bounding.setSize(bounding.size() * ratio)
        bounding.moveTo(bounding.topLeft() * ratio)

        ## draw long line along axis
        pen, p1, p2 = axisSpec
        p.setPen(pen)
        p.drawLine(p1, p2)

        ## draw ticks
        for pen, p1, p2 in tickSpecs:
            p.setPen(pen)
            p.drawLine(p1, p2)

        # Draw all text
        if self.style["tickFont"] is not None:
            p.setFont(self.style["tickFont"])
        p.setPen(self.textPen())

        p.setClipRect(bounding)
        for rect, flags, text in textSpecs:
            p.drawText(rect, int(flags), text)

    def paint(self, p, opt, widget):
        rect = self.boundingRect()

        width = ceil(rect.width())
        height = ceil(rect.height())
        ratio = widget.devicePixelRatio() if widget else 1.0

        if self.picture is None:
            try:
                picture = QtGui.QPixmap(ceil(width * ratio), ceil(height * ratio))

                if self.hovering:
                    if self.background == fn.mkColor("#000000"):
                        bk = fn.mkColor("#323232")
                    else:
                        bk = fn.mkColor("#CDCDCD")

                else:
                    bk = self.background

                picture.fill(bk)

                painter = QtGui.QPainter()
                painter.begin(picture)

                if self.isVisible():
                    painter.setCompositionMode(QtGui.QPainter.CompositionMode.CompositionMode_SourceOver)
                    if self.style["tickFont"]:
                        painter.setFont(self.style["tickFont"])
                    specs = self.generateDrawSpecs(painter, ratio)

                    if specs is not None:
                        self.drawPicture(painter, *specs, ratio)

                    bounding = self.boundingRect().toAlignedRect()
                    bounding.setSize(bounding.size() * ratio)
                    bounding.moveTo(bounding.topLeft() * ratio)

                    if self.minus is not None:
                        painter.drawPixmap(
                            QtCore.QPoint(int(rect.x() * ratio) + 5 * ratio, 6 * ratio),
                            self.minus.pixmap.scaled(BUTTON_SIZE * ratio, BUTTON_SIZE * ratio),
                        )
                        painter.drawPixmap(
                            QtCore.QPoint((int(rect.x()) + 5) * ratio, 27 * ratio),
                            self.plus.pixmap.scaled(BUTTON_SIZE * ratio, BUTTON_SIZE * ratio),
                        )

                    if self.orientation in ("left", "right"):
                        painter.setPen(self._pen)

                        label_rect = QtCore.QRectF(
                            1 * ratio,
                            1 * ratio,
                            rect.height() * ratio - (28 + BUTTON_SIZE) * ratio,
                            rect.width() * ratio,
                        )
                        painter.translate(rect.bottomLeft() * ratio)
                        painter.rotate(-90)

                        painter.setRenderHint(painter.RenderHint.TextAntialiasing, True)
                        painter.drawText(
                            label_rect,
                            QtCore.Qt.AlignmentFlag.AlignHCenter | QtCore.Qt.AlignmentFlag.AlignTop,
                            self._label_with_unit,
                        )
                        painter.rotate(90)
                        painter.resetTransform()

            except:
                print(format_exc())

            finally:
                painter.end()

            picture.setDevicePixelRatio(widget.devicePixelRatio() if widget else 1.0)

            self.picture = picture

    def generateDrawSpecs(self, p, ratio):
        """
        Calls tickValues() and tickStrings() to determine where and how ticks should
        be drawn, then generates from this a set of drawing commands to be
        interpreted by drawPicture().
        """
        bounds = self.mapRectFromParent(self.geometry())

        tickBounds = bounds

        if self.orientation == "left":
            span = (bounds.topRight(), bounds.bottomRight())
            tickStart = tickBounds.right()
            tickStop = bounds.right()
            tickDir = -1
            axis = 0
        elif self.orientation == "right":
            span = (bounds.topLeft(), bounds.bottomLeft())
            tickStart = tickBounds.left()
            tickStop = bounds.left()
            tickDir = 1
            axis = 0
        elif self.orientation == "top":
            span = (bounds.bottomLeft(), bounds.bottomRight())
            tickStart = tickBounds.bottom()
            tickStop = bounds.bottom()
            tickDir = -1
            axis = 1
        elif self.orientation == "bottom":
            span = (bounds.topLeft(), bounds.topRight())
            tickStart = tickBounds.top()
            tickStop = bounds.top()
            tickDir = 1
            axis = 1
        else:
            raise ValueError("self.orientation must be in ('left', 'right', 'top', 'bottom')")

        ## determine size of this item in pixels
        points = list(map(self.mapToDevice, span))
        if None in points:
            return
        lengthInPixels = Point(points[1] - points[0]).length()
        if lengthInPixels == 0:
            return

        # Determine major / minor / subminor axis ticks
        if self._tickLevels is None:
            tickLevels = self.tickValues(self.range[0], self.range[1], lengthInPixels)
            tickStrings = None
        else:
            ## parse self.tickLevels into the formats returned by tickLevels() and tickStrings()
            tickLevels = []
            tickStrings = []
            for level in self._tickLevels:
                values = []
                strings = []
                tickLevels.append((None, values))
                tickStrings.append(strings)
                for val, strn in level:
                    values.append(val)
                    strings.append(strn)

        ## determine mapping between tick values and local coordinates
        dif = self.range[1] - self.range[0]
        if dif == 0:
            xScale = 1
            offset = 0
        else:
            if axis == 0:
                xScale = -bounds.height() / dif
                offset = self.range[0] * xScale - bounds.height()
            else:
                xScale = bounds.width() / dif
                offset = self.range[0] * xScale

        xRange = [x * xScale - offset for x in self.range]
        xMin = min(xRange)
        xMax = max(xRange)

        tickPositions = []  # remembers positions of previously drawn ticks

        ## compute coordinates to draw ticks
        ## draw three different intervals, long ticks first
        tickSpecs = []
        for i in range(len(tickLevels)):
            tickPositions.append([])
            ticks = tickLevels[i][1]

            ## length of tick
            tickLength = self.style["tickLength"] / ((i * 0.5) + 1.0)

            lineAlpha = self.style["tickAlpha"]
            if lineAlpha is None:
                lineAlpha = (0.6**i) * 255

            elif isinstance(lineAlpha, float):
                lineAlpha *= 255
                lineAlpha = max(0, int(round(lineAlpha)))
                lineAlpha = min(255, int(round(lineAlpha)))
            elif isinstance(lineAlpha, int):
                if (lineAlpha > 255) or (lineAlpha < 0):
                    raise ValueError("lineAlpha should be [0..255]")
            else:
                raise TypeError("Line Alpha should be of type None, float or int")

            for v in ticks:
                ## determine actual position to draw this tick
                x = (v * xScale) - offset
                if x < xMin or x > xMax:  ## last check to make sure no out-of-bounds ticks are drawn
                    tickPositions[i].append(None)
                    continue
                tickPositions[i].append(x)

                p1 = [x, x]
                p2 = [x, x]
                p1[axis] = tickStart
                p2[axis] = tickStop
                p2[axis] += tickLength * tickDir
                tickPen = self.pen()
                color = tickPen.color()
                color.setAlpha(int(lineAlpha))
                tickPen.setColor(color)
                tickSpecs.append((tickPen, Point(p1), Point(p2)))

        if self.style["stopAxisAtTick"][0] is True:
            minTickPosition = min(map(min, tickPositions))
            if axis == 0:
                stop = max(span[0].y(), minTickPosition)
                span[0].setY(stop)
            else:
                stop = max(span[0].x(), minTickPosition)
                span[0].setX(stop)
        if self.style["stopAxisAtTick"][1] is True:
            maxTickPosition = max(map(max, tickPositions))
            if axis == 0:
                stop = min(span[1].y(), maxTickPosition)
                span[1].setY(stop)
            else:
                stop = min(span[1].x(), maxTickPosition)
                span[1].setX(stop)
        axisSpec = (self.pen(), span[0], span[1])

        textOffset = self.style["tickTextOffset"][axis]  ## spacing between axis and text
        # if self.style['autoExpandTextSpace'] is True:
        # textWidth = self.textWidth
        # textHeight = self.textHeight
        # else:
        # textWidth = self.style['tickTextWidth'] ## space allocated for horizontal text
        # textHeight = self.style['tickTextHeight'] ## space allocated for horizontal text

        textSize2 = 0
        lastTextSize2 = 0
        textRects = []
        textSpecs = []  ## list of draw

        # If values are hidden, return early
        if not self.style["showValues"]:
            return (axisSpec, tickSpecs, textSpecs)

        for i in range(min(len(tickLevels), self.style["maxTextLevel"] + 1)):
            ## Get the list of strings to display for this level
            if tickStrings is None:
                spacing, values = tickLevels[i]
                strings = self.tickStrings(values, self.autoSIPrefixScale * self.scale, spacing)
            else:
                strings = tickStrings[i]

            if len(strings) == 0:
                continue

            ## ignore strings belonging to ticks that were previously ignored
            for j in range(len(strings)):
                if tickPositions[i][j] is None:
                    strings[j] = None

            ## Measure density of text; decide whether to draw this level
            rects = []
            for s in strings:
                if s is None:
                    rects.append(None)
                else:
                    br = p.boundingRect(
                        QtCore.QRectF(0, 0, 100, 100),
                        QtCore.Qt.AlignmentFlag.AlignCenter,
                        s,
                    )
                    ## boundingRect is usually just a bit too large
                    ## (but this probably depends on per-font metrics?)
                    br.setHeight(br.height() * 0.8)

                    rects.append(br)
                    textRects.append(rects[-1])

            if len(textRects) > 0:
                ## measure all text, make sure there's enough room
                if axis == 0:
                    textSize = np.sum([r.height() for r in textRects])
                    textSize2 = np.max([r.width() for r in textRects])
                else:
                    textSize = np.sum([r.width() for r in textRects])
                    textSize2 = np.max([r.height() for r in textRects])
            else:
                textSize = 0
                textSize2 = 0

            if i > 0:  ## always draw top level
                ## If the strings are too crowded, stop drawing text now.
                ## We use three different crowding limits based on the number
                ## of texts drawn so far.
                textFillRatio = float(textSize) / lengthInPixels
                finished = False
                for nTexts, limit in self.style["textFillLimits"]:
                    if len(textSpecs) >= nTexts and textFillRatio >= limit:
                        finished = True
                        break
                if finished:
                    break

            lastTextSize2 = textSize2

            # spacing, values = tickLevels[best]
            # strings = self.tickStrings(values, self.scale, spacing)
            # Determine exactly where tick text should be drawn
            for j in range(len(strings)):
                vstr = strings[j]
                if vstr is None:  ## this tick was ignored because it is out of bounds
                    continue
                x = tickPositions[i][j]
                # textRect = p.boundingRect(QtCore.QRectF(0, 0, 100, 100), QtCore.Qt.AlignmentFlag.AlignCenter, vstr)
                textRect = rects[j]
                height = textRect.height()
                width = textRect.width()
                # self.textHeight = height
                offset = max(0, self.style["tickLength"]) + textOffset

                if self.orientation == "left":
                    alignFlags = QtCore.Qt.AlignmentFlag.AlignRight | QtCore.Qt.AlignmentFlag.AlignVCenter
                    rect = QtCore.QRectF(tickStop - offset - width, x - (height / 2), width, height)
                elif self.orientation == "right":
                    alignFlags = QtCore.Qt.AlignmentFlag.AlignLeft | QtCore.Qt.AlignmentFlag.AlignVCenter
                    rect = QtCore.QRectF(tickStop + offset, x - (height / 2), width, height)
                elif self.orientation == "top":
                    alignFlags = QtCore.Qt.AlignmentFlag.AlignHCenter | QtCore.Qt.AlignmentFlag.AlignBottom
                    rect = QtCore.QRectF(x - width / 2.0, tickStop - offset - height, width, height)
                elif self.orientation == "bottom":
                    alignFlags = QtCore.Qt.AlignmentFlag.AlignHCenter | QtCore.Qt.AlignmentFlag.AlignTop
                    rect = QtCore.QRectF(x - width / 2.0, tickStop + offset, width, height)

                textFlags = alignFlags | QtCore.Qt.TextFlag.TextDontClip
                # p.setPen(self.pen())
                # p.drawText(rect, textFlags, vstr)
                textSpecs.append((rect, textFlags, vstr))

        ## update max text size if needed.
        self._updateMaxTextSize(lastTextSize2)

        self.tickSpecs = tickSpecs

        axisSpec = (
            axisSpec[0],
            axisSpec[1] * ratio,
            axisSpec[2] * ratio,
        )

        bounds.setSize(bounds.size() * ratio)

        for spec in textSpecs:
            spec[0].setSize(spec[0].size() * ratio)
            spec[0].moveTo(spec[0].topLeft() * ratio)

        for i, spec in enumerate(tickSpecs):
            tickSpecs[i] = (
                spec[0],
                spec[1] * ratio,
                spec[2] * ratio,
            )

        return (axisSpec, tickSpecs, textSpecs)



#!/usr/bin/env python
import sys
import unittest
from unittest import mock

from PySide6 import QtCore, QtGui, QtTest

from test.asammdf.gui.test_base import Pixmap
from test.asammdf.gui.widgets.test_BasePlotWidget import TestPlotWidget


class TestDoubleClick(TestPlotWidget):
    # Note: Test Plot Widget through FileWidget.
    def setUp(self):
        super().setUp()

        # Open File Widget
        self.setUpFileWidget(measurement_file=self.measurement_file, default=True)
        # Switch ComboBox to "Natural sort"
        self.widget.channel_view.setCurrentText("Natural sort")
        # Create a plot window
        self.create_window(window_type="Plot")
        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 1)
        # Drag and Drop channel from FileWidget.channel_tree to Plot
        self.plot = self.widget.mdi_area.subWindowList()[0].widget()

        # Press PushButton 'FocusMode' - disabled (easy for evaluation)
        if not self.plot.focused_mode_btn.isFlat():
            QtTest.QTest.mouseClick(self.plot.focused_mode_btn, QtCore.Qt.MouseButton.LeftButton)
        # Press PushButton "Hide axis"
        if not self.plot.hide_axes_btn.isFlat():
            QtTest.QTest.mouseClick(self.plot.hide_axes_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()

    def test_ChannelSelection(self):
        """
        Test Scope: Validate that doubleClick operation will activate deactivate channels.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
            - Drag and Drop channel from FileWidget.channel_tree to Plot
            - Press mouse double click on added channel
            - Press mouse double click on added channel
        Evaluate:
            - Evaluate that channel checkbox is unchecked.
            - Evaluate that channel checkbox is checked.
        """
        # Event
        plot_channel = self.add_channel_to_plot()

        # Pre-evaluation
        self.assertEqual(QtCore.Qt.CheckState.Checked, plot_channel.checkState(0))

        # Press mouse double click on channel
        self.mouseDClick_WidgetItem(plot_channel)

        # Evaluate
        self.assertEqual(QtCore.Qt.CheckState.Unchecked, plot_channel.checkState(0))

        # Press mouse double click on channel
        self.mouseDClick_WidgetItem(plot_channel)
        self.assertEqual(QtCore.Qt.CheckState.Checked, plot_channel.checkState(0))

    def test_EnableDisable_Group(self):
        """
        Test Scope: Validate that doubleClick operation will activate deactivate groups.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
            - Drag and Drop channels from FileWidget.channel_tree to Plot
            - Press 'Shift-Insert' in order to Insert Group
            - Drag and Drop some channels to new Group
            - Press mouse double click on added group
            - Press key Down few times
            - Press mouse double click on added group
            - Press key Down few times
        Evaluate:
            - Evaluate that group is disabled (not selectable anymore).
            - Evaluate that channel is enabled (selectable).
            - Evaluate that channel is not present on plot when is disabled.
        """
        # Event
        # Drag and Drop channel from FileWidget.channel_tree to Plot
        plot_channel_0 = self.add_channel_to_plot(channel_index=10)
        _ = self.add_channel_to_plot(channel_index=11)
        plot_channel_2 = self.add_channel_to_plot(channel_index=12)
        # Press 'Shift-Insert' in order to Insert Group
        # Create Channel Group. Drag channels inside the group one by one
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog.getText") as mc_getText:
            # Create Channel Group
            mc_getText.return_value = "FirstGroup", True
            QtTest.QTest.keySequence(self.plot.channel_selection, QtGui.QKeySequence("Shift+Insert"))
            self.processEvents()

            # Get Group Position
            for index in range(self.plot.channel_selection.topLevelItemCount()):
                item = self.plot.channel_selection.topLevelItem(index)
                if item.text(self.Column.NAME) == "FirstGroup":
                    first_group = item
                    break
            else:
                self.fail("FirstGroup is not present on Plot Channel Selection.")
            first_group.setExpanded(True)
            # Get the Name of the first channel
            first_channel = plot_channel_0.text(self.Column.NAME)
            # PreEvaluation: Ensure that group has no child
            self.assertEqual(0, first_group.childCount())
            self.move_item_inside_channels_tree_widget(src=plot_channel_0, dst=first_group)
            # PreEvaluate: Ensure that channel was added to group
            self.assertEqual(1, first_group.childCount())
            self.assertEqual(first_channel, first_group.child(0).text(self.Column.NAME))
            self.assertEqual(3, self.plot.channel_selection.topLevelItemCount())

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            # Evaluate
            self.assertTrue(
                Pixmap.has_color(
                    pixmap=enabled_groups_pixmap,
                    color_name=plot_channel_0.color,
                )
            )
            # Press mouse double click on Group
            self.mouseDClick_WidgetItem(first_group)

            # Evaluate
            for _ in range(10):
                self.processEvents(0.05)
            disabled_groups_pixmap = self.plot.plot.viewport().grab()
            self.assertFalse(
                Pixmap.has_color(
                    pixmap=disabled_groups_pixmap,
                    color_name=plot_channel_0.color,
                )
            )

            for _ in range(4):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Evaluate that item is still 2nd one because
            selectedItems = self.plot.channel_selection.selectedItems()
            self.assertEqual(1, len(selectedItems))
            selectedItem = selectedItems[0].text(self.Column.NAME)
            self.assertEqual(plot_channel_2.text(self.Column.NAME), selectedItem)

            # Press mouse double click on channel
            self.mouseDClick_WidgetItem(first_group)
            self.processEvents()

            for _ in range(4):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Evaluate that item is the one from the group
            selectedItems = self.plot.channel_selection.selectedItems()
            self.assertEqual(1, len(selectedItems))
            selectedItem = selectedItems[0].text(self.Column.NAME)
            self.assertEqual(plot_channel_0.text(self.Column.NAME), selectedItem)

            # Evaluate
            # After channels are enabled, they first flash a few times on plot.
            # ProcessEvents few times for channels to be present
            for _ in range(20):
                self.processEvents(0.05)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            self.assertTrue(
                Pixmap.has_color(
                    pixmap=enabled_groups_pixmap,
                    color_name=plot_channel_0.color,
                ),
                msg=f"Color of channel {plot_channel_0.text(self.Column.NAME)} is not present on plot.",
            )

    @unittest.skipIf(sys.platform == "win32", "fails on Windows")
    def test_EnableDisable_ParentGroup(self):
        """
        Test Scope:
        Validate that doubleClick operation will activate deactivate subgroups when parent is disabled.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
            - Drag and Drop channels from FileWidget.channel_tree to Plot
            - Press 'Shift-Insert' in order to Insert Group: A
            - Press 'Shift-Insert' in order to Insert Group: B
            - Press 'Shift-Insert' in order to Insert Group: C
            - Drag and Drop some channels to new Group: A
            - Drag and Drop some channels to new Group: B
            - Drag and Drop some channels to new Group: C
            - Move Group C inside Group B
            - Move Group B inside Group A
            - Press mouse double click on added group: A
            - Press key Down few times
            - Press mouse double click on added group: A
            - Press key Down few times
        Evaluate:
            - Evaluate that subgroups are disabled (not selectable anymore).
            - Evaluate that channels are enabled (selectable).
            - Evaluate that channel is not present on plot when is disabled.
            - Evaluate channel color is darkGray when it's disabled.
        """
        # Event
        # Drag and Drop channel from FileWidget.channel_tree to Plot
        plot_channel_a = self.add_channel_to_plot(channel_index=10)
        plot_channel_b = self.add_channel_to_plot(channel_index=11)
        plot_channel_c = self.add_channel_to_plot(channel_index=12)
        plot_channel_d = self.add_channel_to_plot(channel_index=13)
        self.processEvents()
        # Press 'Shift-Insert' in order to Insert Group
        # Create Channel Group. Drag channels inside the group one by one
        groups = {}
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog.getText") as mc_getText:
            for group_name in ("A", "B", "C"):
                # Create Channel Group
                mc_getText.return_value = group_name, True
                QtTest.QTest.keySequence(self.plot.channel_selection, QtGui.QKeySequence("Shift+Insert"))
                self.processEvents()

            # Get Groups Position
            for index in range(self.plot.channel_selection.topLevelItemCount()):
                item = self.plot.channel_selection.topLevelItem(index)
                group_name = item.text(self.Column.NAME)
                if group_name in ("A", "B", "C"):
                    groups[group_name] = item
                    item.setExpanded(True)

            # Get the First Item that will be moved
            for group_name, plot_channel in zip(
                ("A", "B", "C"), (plot_channel_a, plot_channel_b, plot_channel_c), strict=False
            ):
                # PreEvaluation: Ensure that group has no child
                self.assertEqual(0, groups[group_name].childCount())
                self.move_item_inside_channels_tree_widget(src=plot_channel, dst=groups[group_name])
                self.assertEqual(1, groups[group_name].childCount())
                self.processEvents()

            # Move Group C inside Group B
            self.move_item_inside_channels_tree_widget(src=groups["C"], dst=groups["B"])
            # Move Group B inside Group A
            self.move_item_inside_channels_tree_widget(src=groups["B"], dst=groups["A"])
            groups["A"].setExpanded(True)
            groups["B"].setExpanded(True)
            groups["C"].setExpanded(True)

            self.processEvents()
            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            # Evaluate
            for channel in (plot_channel_a, plot_channel_b, plot_channel_c):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} not present on 'plot'.",
                )
                color_name = channel.foreground(self.Column.NAME).color().name()
                self.assertNotEqual(color_name, "#808080")
            # Press mouse double click on Group A
            self.mouseDClick_WidgetItem(groups["A"])

            # Evaluate
            for _ in range(10):
                self.processEvents(0.05)
            disabled_groups_pixmap = self.plot.plot.viewport().grab()
            for channel in (plot_channel_a, plot_channel_b, plot_channel_c):
                self.assertFalse(
                    Pixmap.has_color(
                        pixmap=disabled_groups_pixmap,
                        color_name=channel.color,
                    ),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} present on 'plot'.",
                )

            QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Up)
            for _ in range(4):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Evaluate that item is still plot_channel_d
            selectedItems = self.plot.channel_selection.selectedItems()
            self.assertEqual(1, len(selectedItems))
            selectedItem = selectedItems[0].text(self.Column.NAME)
            self.assertEqual(plot_channel_d.text(self.Column.NAME), selectedItem)

            # Press mouse double click on channel
            self.mouseDClick_WidgetItem(groups["A"])
            self.processEvents()

            for _ in range(8):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Evaluate that item is the one from the group C
            selectedItems = self.plot.channel_selection.selectedItems()
            self.assertEqual(1, len(selectedItems))
            selectedItem = selectedItems[0].text(self.Column.NAME)
            self.assertEqual(plot_channel_c.text(self.Column.NAME), selectedItem)

            # Evaluate
            # After channels are enabled, they first flash a few times on self.plot.
            # ProcessEvents few times for channels to be present
            for _ in range(20):
                self.processEvents(0.05)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            for channel in (
                plot_channel_a,
                plot_channel_b,
                plot_channel_c,
                plot_channel_d,
            ):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color for Channel: {channel.text(self.Column.NAME)} not present on 'plot'",
                )

    @unittest.skipIf(sys.platform == "win32", "fails on Windows")
    def test_EnableDisable_Subgroup(self):
        """
        Test Scope:
        Validate that doubleClick operation will activate deactivate subgroups and keep parent enabled.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
            - Drag and Drop channels from FileWidget.channel_tree to Plot
            - Press 'Shift-Insert' in order to Insert Group: A
            - Press 'Shift-Insert' in order to Insert Group: B
            - Press 'Shift-Insert' in order to Insert Group: C
            - Drag and Drop some channels to new Group: A
            - Drag and Drop some channels to new Group: B
            - Drag and Drop some channels to new Group: C
            - Move Group C inside Group B
            - Move Group B inside Group A
            - Press mouse double click on added group: B
            - Press key Down few times
            - Press mouse double click on added group: B
            - Press key Down few times
        Evaluate:
            - Evaluate that subgroups are disabled (not selectable anymore).
            - Evaluate that channels are enabled (selectable).
            - Evaluate that channel is not present on plot when is disabled.
        """
        # Event
        # Drag and Drop channel from FileWidget.channel_tree to Plot
        plot_channel_a = self.add_channel_to_plot(channel_index=10)
        plot_channel_b = self.add_channel_to_plot(channel_index=11)
        plot_channel_c = self.add_channel_to_plot(channel_index=12)
        plot_channel_d = self.add_channel_to_plot(channel_index=13)

        # Press 'Shift-Insert' in order to Insert Group
        # Create Channel Group. Drag channels inside the group one by one
        groups = {}
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog.getText") as mc_getText:
            for group_name in ("A", "B", "C"):
                # Create Channel Group
                mc_getText.return_value = group_name, True
                QtTest.QTest.keySequence(self.plot.channel_selection, QtGui.QKeySequence("Shift+Insert"))
                self.processEvents()

            # Get Groups Position
            for index in range(self.plot.channel_selection.topLevelItemCount()):
                item = self.plot.channel_selection.topLevelItem(index)
                group_name = item.text(self.Column.NAME)
                if group_name in ("A", "B", "C"):
                    groups[group_name] = item
                    item.setExpanded(True)

            # Get the First Item that will be moved
            for group_name, plot_channel in zip(
                ("A", "B", "C"), (plot_channel_a, plot_channel_b, plot_channel_c), strict=False
            ):
                # PreEvaluation: Ensure that group has no child
                self.assertEqual(0, groups[group_name].childCount())
                self.move_item_inside_channels_tree_widget(src=plot_channel, dst=groups[group_name])
                self.assertEqual(1, groups[group_name].childCount())
                self.processEvents()

            # Move Group C inside Group B
            self.move_item_inside_channels_tree_widget(src=groups["C"], dst=groups["B"])
            # Move Group B inside Group A
            self.move_item_inside_channels_tree_widget(src=groups["B"], dst=groups["A"])

            groups["A"].setExpanded(True)
            groups["B"].setExpanded(True)
            groups["C"].setExpanded(True)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            self.processEvents()
            # Evaluate
            for channel in (plot_channel_a, plot_channel_b, plot_channel_c):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} not present on 'plot'.",
                )
            # Press mouse double click on Group B
            self.mouseDClick_WidgetItem(groups["B"])
            disabled_groups_pixmap = self.plot.plot.viewport().grab()

            # Evaluate
            self.assertTrue(
                Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=plot_channel_a.color),
                msg=f"Color of Channel: {plot_channel_a.text(self.Column.NAME)} not present on 'plot'.",
            )
            for channel in (plot_channel_b, plot_channel_c):
                self.assertFalse(
                    Pixmap.has_color(
                        pixmap=disabled_groups_pixmap,
                        color_name=channel.color,
                    ),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} present on 'plot'.",
                )

            QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Up)
            for _ in range(4):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Press mouse double click on group B
            self.mouseDClick_WidgetItem(groups["B"])
            self.processEvents()

            for _ in range(8):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Evaluate
            # After channels are enabled, they first flash a few times on self.plot.
            # ProcessEvents few times for channels to be present
            for _ in range(20):
                self.processEvents(0.05)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            for channel in (
                plot_channel_a,
                plot_channel_b,
                plot_channel_c,
                plot_channel_d,
            ):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color for Channel: {channel.text(self.Column.NAME)} not present on 'plot'",
                )

    def test_EnableDisable_Preserve_Subgroup_State_0(self):
        """
        Test Scope:
            Validate that doubleClick operation will activate/deactivate subgroups and preserve state
            when parent is disabled/enabled.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
            - Drag and Drop channels from FileWidget.channel_tree to Plot
            - Press 'Shift-Insert' in order to Insert Group: A
            - Press 'Shift-Insert' in order to Insert Group: B
            - Press 'Shift-Insert' in order to Insert Group: C
            - Drag and Drop some channels to new Group: A
            - Drag and Drop some channels to new Group: B
            - Drag and Drop some channels to new Group: C
            - Move Group C inside Group B
            - Move Group B inside Group A
            - Press mouse double click on added group: C
            - Press mouse double click on added group: B
            - Press key Down few times
            - Press mouse double click on added group: B
            - Press key Down few times
        Evaluate:
            - Evaluate that subgroups are disabled (not selectable anymore).
            - Evaluate that channel is not present on plot when is disabled.
            - Evaluate that subgroup C state is maintained after subgroup B is enabled.
        """
        # Event
        # Drag and Drop channel from FileWidget.channel_tree to Plot
        plot_channel_a = self.add_channel_to_plot(channel_index=10)
        plot_channel_b = self.add_channel_to_plot(channel_index=11)
        plot_channel_c = self.add_channel_to_plot(channel_index=12)
        plot_channel_d = self.add_channel_to_plot(channel_index=13)
        # Press 'Shift-Insert' in order to Insert Group
        # Create Channel Group. Drag channels inside the group one by one
        groups = {}
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog.getText") as mc_getText:
            for group_name in ("A", "B", "C"):
                # Create Channel Group
                mc_getText.return_value = group_name, True
                QtTest.QTest.keySequence(self.plot.channel_selection, QtGui.QKeySequence("Shift+Insert"))
                self.processEvents()

            # Get Groups Position
            for index in range(self.plot.channel_selection.topLevelItemCount()):
                item = self.plot.channel_selection.topLevelItem(index)
                group_name = item.text(self.Column.NAME)
                if group_name in ("A", "B", "C"):
                    groups[group_name] = item
                    item.setExpanded(True)

            # Get the First Item that will be moved
            for group_name, plot_channel in zip(
                ("A", "B", "C"), (plot_channel_a, plot_channel_b, plot_channel_c), strict=False
            ):
                # PreEvaluation: Ensure that group has no child
                self.assertEqual(0, groups[group_name].childCount())
                self.move_item_inside_channels_tree_widget(src=plot_channel, dst=groups[group_name])

                self.processEvents(0.05)
                self.assertEqual(1, groups[group_name].childCount())

            # Move Group C inside Group B
            self.move_item_inside_channels_tree_widget(src=groups["C"], dst=groups["B"])
            # Move Group B inside Group A
            self.move_item_inside_channels_tree_widget(src=groups["B"], dst=groups["A"])

            groups["A"].setExpanded(True)
            groups["B"].setExpanded(True)
            groups["C"].setExpanded(True)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            self.processEvents()
            # Evaluate
            for channel in (plot_channel_a, plot_channel_b, plot_channel_c):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} not present on 'plot'.",
                )
            # Press mouse double click on Group C
            self.mouseDClick_WidgetItem(groups["C"])
            # Press mouse double click on Group B
            self.mouseDClick_WidgetItem(groups["B"])
            disabled_groups_pixmap = self.plot.plot.viewport().grab()

            # Evaluate
            self.assertTrue(
                Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=plot_channel_a.color),
                msg=f"Color of Channel: {plot_channel_a.text(self.Column.NAME)} not present on 'plot'.",
            )
            for channel in (plot_channel_b, plot_channel_c):
                self.assertFalse(
                    Pixmap.has_color(
                        pixmap=disabled_groups_pixmap,
                        color_name=channel.color,
                    ),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} present on 'plot'.",
                )

            QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Up)
            for _ in range(4):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Press mouse double click on group B
            self.mouseDClick_WidgetItem(groups["B"])
            self.processEvents()

            for _ in range(8):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Evaluate
            # After channels are enabled, they first flash a few times on self.plot.
            # ProcessEvents few times for channels to be present
            for _ in range(20):
                self.processEvents(0.05)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            self.assertFalse(
                Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=plot_channel_c.color),
                msg=f"Color for Channel: {plot_channel_c.text(self.Column.NAME)} present on 'plot'",
            )
            for channel in (plot_channel_a, plot_channel_b, plot_channel_d):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color for Channel: {channel.text(self.Column.NAME)} not present on 'plot'",
                )

    def test_EnableDisable_Preserve_Subgroup_State_1(self):
        """
        Test Scope:
            Validate that doubleClick operation will activate/deactivate subgroups and preserve state
            when parent is disabled/enabled.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
            - Drag and Drop channels from FileWidget.channel_tree to Plot
            - Press 'Shift-Insert' in order to Insert Group: A
            - Press 'Shift-Insert' in order to Insert Group: B
            - Press 'Shift-Insert' in order to Insert Group: C
            - Drag and Drop some channels to new Group: A
            - Drag and Drop some channels to new Group: B
            - Drag and Drop some channels to new Group: C
            - Move Group C inside Group B
            - Move Group B inside Group A
            - Press mouse double click on added group: C
            - Press mouse double click on added group: B
            - Press key Down few times
            - Press mouse double click on added group: B
            - Press mouse click on group B:
                - Use supposed to simulate Deactivate group from ContextMenu for group: C
                but the bug is related to previous enabled group which remains selected.
            - Press key Down few times
        Evaluate:
            - Evaluate that subgroups are disabled (not selectable anymore).
            - Evaluate that channels are enabled (selectable).
            - Evaluate that channel is not present on plot when is disabled.
        """
        # Event
        # Drag and Drop channel from FileWidget.channel_tree to Plot
        plot_channel_a = self.add_channel_to_plot(channel_index=10)
        plot_channel_b = self.add_channel_to_plot(channel_index=11)
        plot_channel_c = self.add_channel_to_plot(channel_index=12)
        plot_channel_d = self.add_channel_to_plot(channel_index=13)
        # Press 'Shift-Insert' in order to Insert Group
        # Create Channel Group. Drag channels inside the group one by one
        groups = {}
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog.getText") as mc_getText:
            for group_name in ("A", "B", "C"):
                # Create Channel Group
                mc_getText.return_value = group_name, True
                QtTest.QTest.keySequence(self.plot.channel_selection, QtGui.QKeySequence("Shift+Insert"))
                self.processEvents()

            # Get Groups Position
            for index in range(self.plot.channel_selection.topLevelItemCount()):
                item = self.plot.channel_selection.topLevelItem(index)
                group_name = item.text(self.Column.NAME)
                if group_name in ("A", "B", "C"):
                    groups[group_name] = item
                    item.setExpanded(True)

            # Get the First Item that will be moved
            for group_name, plot_channel in zip(
                ("A", "B", "C"), (plot_channel_a, plot_channel_b, plot_channel_c), strict=False
            ):
                # PreEvaluation: Ensure that group has no child
                self.assertEqual(0, groups[group_name].childCount())
                self.move_item_inside_channels_tree_widget(src=plot_channel, dst=groups[group_name])

                self.processEvents(0.05)
                self.assertEqual(1, groups[group_name].childCount())

            # Move Group C inside Group B
            self.move_item_inside_channels_tree_widget(src=groups["C"], dst=groups["B"])
            # Move Group B inside Group A
            self.move_item_inside_channels_tree_widget(src=groups["B"], dst=groups["A"])

            groups["A"].setExpanded(True)
            groups["B"].setExpanded(True)
            groups["C"].setExpanded(True)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            self.processEvents()
            # Evaluate
            for channel in (plot_channel_a, plot_channel_b, plot_channel_c):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} not present on 'plot'.",
                )
            # Press mouse double click on Group C
            self.mouseDClick_WidgetItem(groups["C"])
            # Press mouse double click on Group B
            self.mouseDClick_WidgetItem(groups["B"])
            disabled_groups_pixmap = self.plot.plot.viewport().grab()

            # Evaluate
            self.assertTrue(
                Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=plot_channel_a.color),
                msg=f"Color of Channel: {plot_channel_a.text(self.Column.NAME)} not present on 'plot'.",
            )
            for channel in (plot_channel_b, plot_channel_c):
                self.assertFalse(
                    Pixmap.has_color(
                        pixmap=disabled_groups_pixmap,
                        color_name=channel.color,
                    ),
                    msg=f"Color of Channel: {channel.text(self.Column.NAME)} present on 'plot'.",
                )

            QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Up)
            for _ in range(4):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Press mouse double click on group B
            self.mouseDClick_WidgetItem(groups["B"])
            self.mouseClick_WidgetItem(groups["B"])
            self.processEvents()

            # Evaluate selected items
            selectedItems = self.plot.channel_selection.selectedItems()
            self.assertEqual(1, len(selectedItems))
            self.assertEqual("B", selectedItems[0].text(self.Column.NAME))

            # Disable group C from context-menu
            # Disclaimer: Group C is already disabled
            # Simple Click on group C is enough. The bug was that when you click disabled item,
            # the previous selected item remains in selectedItems.
            # Basically, the disabled item is not selectable and selection is kept.
            # In case that previous item was a group, and 'Disable groups' is selected from the context-menu
            # then, unintended parent and subgroups disable will happen.
            self.mouseClick_WidgetItem(groups["C"])

            # Evaluate that selection was changed.
            selectedItems = self.plot.channel_selection.selectedItems()
            self.assertEqual(0, len(selectedItems))

            for _ in range(8):
                QtTest.QTest.keyClick(self.plot.channel_selection, QtCore.Qt.Key.Key_Down)
                self.processEvents()

            # Evaluate
            # After channels are enabled, they first flash a few times on self.plot.
            # ProcessEvents few times for channels to be present
            for _ in range(20):
                self.processEvents(0.05)

            enabled_groups_pixmap = self.plot.plot.viewport().grab()
            self.assertFalse(
                Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=plot_channel_c.color),
                msg=f"Color for Channel: {plot_channel_c.text(self.Column.NAME)} present on 'plot'",
            )
            for channel in (plot_channel_a, plot_channel_b, plot_channel_d):
                self.assertTrue(
                    Pixmap.has_color(pixmap=enabled_groups_pixmap, color_name=channel.color),
                    msg=f"Color for Channel: {channel.text(self.Column.NAME)} not present on 'plot'",
                )



#!/usr/bin/env python\

from os import path
from pathlib import Path
from sys import platform
from unittest import mock, skipIf

import numpy
from PySide6 import QtGui, QtTest
from PySide6.QtCore import QPoint, QRect
from PySide6.QtWidgets import QApplication

from asammdf import mdf
from test.asammdf.gui.test_base import Pixmap
from test.asammdf.gui.widgets.test_BaseFileWidget import TestFileWidget


class TestDataTableViewShortcuts(TestFileWidget):
    def setUp(self):
        """
        Events:
            - Open valid measurement file in ASAMMDF
            - Create a Tabular window with two channels on it

        Evaluate:
            - Evaluate that is one active sub-window
        Returns
        -------

        """
        super().setUp()
        # Open measurement file
        measurement_file = str(Path(self.resource, "ASAP2_Demo_V171.mf4"))

        # Open measurement file
        self.setUpFileWidget(measurement_file=measurement_file, default=True)
        # Create a tabular window
        self.create_window(window_type="Tabular", channels_indexes=(35, 36))
        # Evaluate
        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 1)
        self.tabular = self.widget.mdi_area.subWindowList()[0].widget()
        self.dtw = self.tabular.tree.dataView
        # Load shortcuts
        self.assertIsNotNone(self.load_shortcuts_from_json_file(self.dtw))
        self.processEvents(0.01)

    def test_set_color_range_shortcut(self):
        """
            Test Scope:
                Ensure that after pressing key Ctrl+R, range editor will be triggered

            Events:
                - Click on first channel
                - Press Ctrl+R
                - Add range
                - Ok

            Evaluate:
                - Evaluate that range editor was triggered after pressing key Ctrl+R
                - Evaluate that range was applied for only for selected signal
                - Evaluate that only cells with value within range are colored
        Returns
        -------

        """
        # Setup
        col = 1
        value1 = 0.0
        value2 = 150.0
        self.processEvents()
        green = QtGui.QColor.fromRgbF(0.000000, 1.000000, 0.000000, 1.000000)
        green_brush = QtGui.QBrush(green, QtGui.Qt.SolidPattern)
        red = QtGui.QColor.fromRgbF(1.000000, 0.000000, 0.000000, 1.000000)
        red_brush = QtGui.QBrush(red, QtGui.Qt.SolidPattern)
        range_editor_result = [
            {
                "background_color": green_brush,
                "font_color": red_brush,
                "op1": "<",
                "op2": "<",
                "value1": value1,
                "value2": value2,
            }
        ]

        # Select first row
        self.dtw.selectColumn(col)
        # QtTest.QTest.keySequence(self.dtw, QtGui.QKeySequence(self.shortcuts["set_color_range"]))
        self.processEvents()
        with mock.patch("asammdf.gui.widgets.tabular_base.RangeEditor") as mo_RangeEditor:
            mo_RangeEditor.return_value.result = range_editor_result
            mo_RangeEditor.return_value.pressed_button = "apply"
            # Press "Ctrl+R"
            QtTest.QTest.keySequence(self.dtw, QtGui.QKeySequence(self.shortcuts["set_color_range"]))
            self.processEvents()
        self.dtw.selectColumn(col)
        self.processEvents(0.01)
        # Evaluate
        mo_RangeEditor.assert_called()
        col_0 = True
        for value in self.tabular.ranges.values():
            if col_0 is True:  # Evaluate range for first row
                self.assertDictEqual(value[0], range_editor_result[0])
                col_0 = False
            else:
                self.assertListEqual(value, [])

        # Evaluate cell color
        row = 0
        # Find regular cell
        for row in range(self.dtw.pgdf.df.values.size - 1):
            if self.dtw.pgdf.df.values[row][1] > value2:
                break

                # GoTo regular row
        self.dtw.selectRow(row)
        self.dtw.selectRow(row)
        self.processEvents(0.01)

        # Cell to evaluate
        pm = self.dtw.grab(
            QRect(
                self.dtw.columnViewportPosition(col),
                self.dtw.rowViewportPosition(row),
                self.dtw.columnWidth(col),
                self.dtw.rowHeight(row),
            )
        )
        # Evaluate
        self.assertFalse(Pixmap.has_color(pm, red))
        self.assertFalse(Pixmap.has_color(pm, green))

        # Find colored row
        for row in range(self.dtw.pgdf.df.values.size - 1):
            if value1 < self.dtw.pgdf.df.values[row][1] < value2:
                break

        # GoTo colored row
        self.dtw.selectRow(row)
        self.dtw.selectRow(row)
        self.processEvents(0.01)

        # Cell to evaluate
        pm = self.dtw.grab(
            QRect(
                self.dtw.columnViewportPosition(col),
                self.dtw.rowViewportPosition(row),
                self.dtw.columnWidth(col),
                self.dtw.rowHeight(row),
            )
        )
        # Evaluate
        self.assertTrue(Pixmap.has_color(pm, green))
        self.assertTrue(Pixmap.has_color(pm, red))


class TestTabularBaseShortcuts(TestFileWidget):
    def setUp(self):
        """
        Events:
            - Open valid measurement file in ASAMMDF
            - Create a Tabular window with two channels on it

        Evaluate:
            - Evaluate that is one active sub-window
        Returns
        -------

        """
        super().setUp()
        # Open measurement file
        measurement_file = str(Path(self.resource, "ASAP2_Demo_V171.mf4"))

        # Open measurement file
        self.setUpFileWidget(measurement_file=measurement_file, default=True)

        self.create_window(window_type="Tabular", channels_indexes=(35, 36))

        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 1)
        self.tabular = self.widget.mdi_area.subWindowList()[0].widget()
        self.assertIsNotNone(self.load_shortcuts_from_json_file(self.tabular))
        self.processEvents(0.01)

    def test_bin__hex__physical_shortcuts(self):
        """
            Test scope:
                Ensure that data format is changed after pressing keys
                    - Ctrl+B -> to binary
                    - Ctrl+H -> to hexadecimal
                    - Ctrl+P -> to physical

            Events:
                - Press Ctrl+H
                - Press Ctrl+B
                - Press Ctrl+P

            Evaluate:
                - Evaluate that data format and value were changed to bin after pressing key Ctrl+H
                - Evaluate that data format and value were changed to hex after pressing key Ctrl+B
                - Evaluate that data format and value were changed to physical after pressing key Ctrl+P

        Returns
        -------

        """
        # Setup
        self.tabular.set_format("phys")
        data_view = self.tabular.tree.dataView
        # index for first cell of first signal
        index = data_view.indexAt(QPoint(data_view.columnViewportPosition(1), data_view.rowViewportPosition(0)))
        self.processEvents(0.01)
        # Store value from cell
        data = data_view.model().data(index)

        # Event
        # Press Ctrl+H
        QtTest.QTest.keySequence(self.tabular, QtGui.QKeySequence(self.shortcuts["hex"]))
        self.processEvents(0.1)
        # Store value from cell
        hex_data = data_view.model().data(index)

        # Evaluate
        self.assertEqual(self.tabular.format, "hex")
        self.assertIn("0x", hex_data)
        self.assertEqual(int(hex_data, 16), int(data))

        # Event
        # Press Ctrl+B
        QtTest.QTest.keySequence(self.tabular, QtGui.QKeySequence(self.shortcuts["bin"]))
        self.processEvents(0.1)
        # Store value from cell
        bin_data = data_view.model().data(index)

        # Evaluate
        self.assertEqual(self.tabular.format, "bin")
        self.assertIn(".", bin_data)
        self.assertEqual(int(bin_data.replace(".", ""), 2), int(data))

        # Event
        # Press Ctrl+P
        QtTest.QTest.keySequence(self.tabular, QtGui.QKeySequence(self.shortcuts["physical"]))
        self.processEvents(0.1)
        # Store value from cell
        phys_data = data_view.model().data(index)

        # Evaluate
        self.assertEqual(self.tabular.format, "phys")
        self.assertTrue(phys_data.isdecimal())
        self.assertEqual(int(phys_data), int(data))

    def test_save_active_subplot_channels_shortcut(self):
        """
            Test scope:
                Ensure that window Save as file was called
                    and all active subplots were saved into a new file

            Events:
                - Open second sub-window
                - Select first one
                - Press Ctrl+S
                - Set new name for file and click "Save" button

            Evaluate:
                - Evaluate that there are two active sub-windows
                - Evaluate that "Save as file" window was called
                - Evaluate that in new saved file is only channels from selected active sub-window
        Returns
        -------

        """
        # Setup
        self.create_window(window_type="Tabular", channels_indexes=(10, 11))
        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 2)
        self.processEvents()
        expected_channels = ["time"]
        for key in self.tabular.ranges.keys():
            expected_channels.append(key)

        file_path = path.join(self.test_workspace, "file.mf4")
        # mock for getSaveFileName object
        with mock.patch("asammdf.gui.widgets.tabular_base.QtWidgets.QFileDialog.getSaveFileName") as mo_getSaveFileName:
            mo_getSaveFileName.return_value = (file_path, "")
            # Press Ctrl+S
            QtTest.QTest.keySequence(self.tabular, QtGui.QKeySequence(self.shortcuts["save_active_subplot_channels"]))
        # Evaluate
        mo_getSaveFileName.assert_called()

        # get saved file as MDF
        process_bus_logging = ("process_bus_logging", True)
        mdf_file = mdf.MDF(file_path, process_bus_logging=process_bus_logging)

        # Evaluate
        for name in mdf_file.channels_db.keys():
            self.assertIn(name, expected_channels)
            expected_channels.remove(name)
        self.assertEqual(len(expected_channels), 0)
        mdf_file.close()

    def test_increase__decrease_font_shortcuts(self):
        """
        Test scope:
            Ensure that Ctrl+[ and Ctrl+] will change font size

        Events:
            - Press Ctrl+[
            - Press Ctrl+]

        Evaluate:
         - Evaluate that font size was increased after shortcut Ctrl+[ was pressed
         - Evaluate that font size was decreased after shortcut Ctrl+] was pressed
        """
        font_size = self.tabular.tree.dataView.font().pointSize()
        QtTest.QTest.keySequence(self.tabular, QtGui.QKeySequence(self.shortcuts["decrease_font"]))
        self.assertLess(font_size, self.tabular.tree.dataView.font().pointSize())

        font_size = self.tabular.tree.dataView.font().pointSize()
        QtTest.QTest.keySequence(self.tabular, QtGui.QKeySequence(self.shortcuts["increase_font"]))
        self.assertGreater(font_size, self.tabular.tree.dataView.font().pointSize())


@skipIf(platform != "win32", "Failed on linux. Shortcut can copy only value for one cell")
class TestDataFrameViewerShortcuts(TestFileWidget):
    def setUp(self):
        """
            Events:
                - Open valid measurement file in ASAMMDF
                - Create a Tabular window with two channels on it

            Evaluate:
                - Evaluate that is one active sub-window
        Returns
        -------

        """
        super().setUp()
        # Open measurement file
        measurement_file = str(Path(self.resource, "ASAP2_Demo_V171.mf4"))

        # Open measurement file
        self.setUpFileWidget(measurement_file=measurement_file, default=True)

        self.create_window(window_type="Tabular", channels_indexes=(35, 36))

        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 1)
        self.tabular = self.widget.mdi_area.subWindowList()[0].widget()
        self.dfw = self.tabular.tree
        self.assertIsNotNone(self.load_shortcuts_from_json_file(self.dfw))
        self.processEvents(0.01)

    def test_copy_shortcut(self):
        """
            Test scope:
                Ensure that Ctrl+C shortcut will copy value from selected row to clipboard

            Events:
                - Select one row.
                - Press Ctrl+C.

            Evaluate:
             - Evaluate that value from the selected row was copied to clipboard after shortcut Ctrl+C was pressed.
        Returns
        -------

        """

        def copy_header():
            df = self.dfw.pgdf.df
            copied_values = ""
            for name in df.columns:
                copied_values += "\t" + name
            return copied_values + "\n"

        def copy_row(r):
            df = self.dfw.pgdf.df
            fp = self.dfw.dataView.model().float_precision
            copied_values = ""
            for name in df.columns:
                x = df[name][r]
                # if x == r:
                #     continue
                if isinstance(x, numpy.floating):
                    if fp != -1:
                        x = f"\t{x:.16f}"
                    else:
                        x = f"\t{x:.16f}"
                else:
                    x = f"\t{x}"
                copied_values += x

            return copied_values + "\n"

        # -|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|-

        # Setup
        row = 0
        expected_data = copy_header() + "0" + copy_row(row)

        # Event
        self.dfw.dataView.selectRow(row)
        self.processEvents(0.1)
        QtTest.QTest.keySequence(self.dfw, QtGui.QKeySequence(self.shortcuts["copy"]))
        self.processEvents(1)
        data = QApplication.instance().clipboard().text()

        # Test failed, because of clipboard buffer on linux
        # Evaluate
        self.assertEqual(expected_data, data)

        # -|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|-

        # Setup 2
        expected_data = copy_header()
        for row in range(len(self.dfw.pgdf.df.values)):
            expected_data += str(row) + copy_row(row)

        # Event 2
        QtTest.QTest.keySequence(self.dfw, QtGui.QKeySequence("Ctrl+A"))
        self.processEvents()
        QtTest.QTest.keySequence(self.dfw, QtGui.QKeySequence(self.shortcuts["copy"]))
        self.processEvents(1)

        data = QApplication.instance().clipboard().text()

        # Evaluate
        self.assertEqual(expected_data, data)
